#include <set>
#include <sf/core/TextureDeclarationsLoader.h>
#include <sf/core/Declaration.h>
#include <sf/misc/string.h>
#include <sf/graphics/Definitions.h>

#include "AndroidApp.h"

#ifdef __SF_ANDROID
#include <sf/core/android/AndroidApplication.h>
#include <sf/core/android/jni.h>
#endif

namespace game
{

#ifdef __SF_ANDROID
static jclass g_MainActivityClass = 0;
static jmethodID g_OpenSubscription = 0;
#endif

CAndroidApp::CAndroidApp()
{

#ifdef __SF_ANDROID
    SFASSERT(sf::core::g_JavaVM);

	g_MainActivityClass = sf::core::JNIGetClass("com/stargaze/app/MainActivity");
	g_OpenSubscription = sf::core::JNIGetStaticMethod(g_MainActivityClass, "subscribe", "(Ljava/lang/String;)V");
#endif

}

CAndroidApp::~CAndroidApp()
{
}

	
void CAndroidApp::ShowLink(const char * _link)
{
#ifdef __SF_ANDROID
	sf::core::CAndroidApplication* app =
		(sf::core::CAndroidApplication*)sf::core::g_Application->GetSystemIntegration();
	app->ShowLink(_link);
#endif
}

void CAndroidApp::Subscription()
{
#ifdef __SF_ANDROID
	std::string name(sf::core::g_Application->GetSettings().GetChild("settings", false).GetValue("product"));
	jstring jname = sf::core::g_JavaEnv->NewStringUTF(name.c_str());
	sf::core::g_JavaEnv->CallStaticObjectMethod(g_MainActivityClass, g_OpenSubscription, jname);
	sf::core::g_JavaEnv->DeleteLocalRef(jname);
#endif
}

int CAndroidApp::GetMemoryUsage()
{
	static int usage = 0;

#ifdef __SF_ANDROID
	static int counter = 20;

	if(counter++ >= 20)
	{
		char buf[256];
		sprintf(buf, "/proc/%d/status", getpid());
		FILE * fp = fopen(buf, "rb");
		if(fp != NULL)
		{
			while(!feof(fp))
			{
				fgets(buf, sizeof(buf), fp);
				if(strstr(buf, "VmRSS:"))
				{
					char * s = buf + 6;
					while(*s == ' ' || *s == '\t') ++s;
					sscanf(s, "%d", &usage);
					break;
				}
			}
		}		
		fclose(fp);
		counter = 0;
	}
#endif // __SF_ANDROID

	return usage;
}

int CAndroidApp::GetFontTexturesMemoryAmount()
{
	using namespace sf::core;
	using namespace sf;

	const GroupDeclarations<FontDeclaration> & fd = GetResourceDeclarations().GetGroupFontDeclarations();

	const CTextureDeclarationsLoader::SingleStorageHead * h = g_TextureDeclarationsLoader::Instance().GetData();
	SFASSERT(h);

	std::set<IdString> ids;
	int amount = 0;
	for(GroupDeclarations<FontDeclaration>::DeclarationsT::const_iterator it = fd.GetDeclarations().begin()
		, end = fd.GetDeclarations().end(); it != end; ++it)
	{
		if(!ids.count(it->Image))
		{
			ids.insert(it->Image);

			const CTextureDeclarationsLoader::SingleStorageHead::TextureTag * tag = h->FindTextureById(it->Image);
			SFASSERT(tag);
			if(tag)
			{
				const CTextureDeclarationsLoader::TextureDesc * texture = h->GetTexture(tag->texture_offset);
				SFASSERT(texture);
				if(texture)
				{
					const CTextureDeclarationsLoader::SurfaceDesc * surface  = h->GetSurface(texture->surface_offset);
					SFASSERT(surface);

					int bytes = 2;
					switch(surface->internal_format)
					{
					case sf::graphics::InternalFormatBGRA8:
					case sf::graphics::InternalFormatRGBA8:
					case sf::graphics::InternalFormatDXT3:
					case sf::graphics::InternalFormatDXT5:
						bytes = 4;
						break;
					case sf::graphics::InternalFormatPVRTC2:
						bytes = 1;
						break;
					}

					amount += surface->height * surface->width * bytes;
				}
			}
		}
	}
	return amount;
}

}