#include <sf/sf.h>
#include "Branch.h"

#include <sf/core/application.h>
#include <sf/core/texturemanager.h>
#include <sf/gui/buttonwidget.h>
#include <sf/gui/imagewidget.h>
#include <sf/graphics/texture.h>
#include <sf/misc/sys/path.h>
#include <sf/misc/sys/filewrp.h>

namespace game
{
///////////////////////////////////////////////////////////////////
// class CBranch
///////////////////////////////////////////////////////////////////

CBranch::CBranch() :
	sf::gui::CWidget(sf::IdString("branch"), 0, BroadcastOnMouseDown),
    m_SplashEnded(false),
    m_CurrentSplashTime(0.f),
    m_CurrentSplash(new sf::gui::CImageWidget)
{
	LoadSplash("branch_logo1");
	LoadSplash("branch_logo2");
	LoadSplash("branch_logo3");
	LoadSplash("branch_logo4");
	LoadSplash("branch_logo5");

    AddWidget(m_CurrentSplash);

    ShowSplash();
}

CBranch::~CBranch()
{
}

bool CBranch::SplashFinished()
{
    return m_SplashEnded;
}

void CBranch::DoUpdate()
{
    CWidget::DoUpdate();

    if (!m_SplashEnded)
    {
        if (m_Splashes.empty())
        {
            m_CurrentSplash->SetColor(0);
            m_SplashEnded = true;
        }
        else
        {
            float time = sf::core::g_TimeManager::Instance().GetTime() - m_CurrentSplashTime;
            if (time >= m_Splashes.front().Time)
            {
                m_Splashes.pop_front();
                ShowSplash();
            }
            else
            {
                unsigned alpha = 255;
                if (time < m_Splashes.front().ShowTime)
                {
                    alpha = sf::misc::clamp(
                        unsigned(255.f * time / m_Splashes.front().ShowTime), 0u, 255u);
                }
                else if (time > m_Splashes.front().Time - m_Splashes.front().ShowTime)
                {
                    alpha = sf::misc::clamp(unsigned(255.f - 255.f *
                                                     (time -
                                                      (m_Splashes.front().Time -
                                                       m_Splashes.front().ShowTime)) /
                                                     m_Splashes.front().ShowTime), 0u, 255u);
                }
                m_CurrentSplash->SetColor((alpha << 24) | 0xffffffu);
            }
        }
    }
}

void CBranch::DoDraw(sf::graphics::CRenderer* _renderer) const
{
    if (!m_SplashEnded)
        _renderer->SetColor(0xffffffffu);
    CWidget::DoDraw(_renderer);
}

void CBranch::ShowSplash()
{
    while (!m_Splashes.empty())
    {
		m_CurrentSplash->SetImage(*m_Splashes.front().m_Image);
        m_CurrentSplash->SetColor(0x00ffffffu);

        if (!m_CurrentSplash->GetImage().GetTexture() 
            || m_CurrentSplash->GetImage().GetTexture()->GetWidth() < 3)
        {
            m_Splashes.pop_front();
        }
        else
        {
            m_CurrentSplash->SetSize(
                float(m_CurrentSplash->GetImage().GetTexture()->GetWidth()),
                float(m_CurrentSplash->GetImage().GetTexture()->GetHeight()));

            m_CurrentSplash->SetOffset(
                (sf::core::g_Application->GetRenderer()->GetSourceResolution().X -
                 m_CurrentSplash->GetSize().X) / 2.f,
                (sf::core::g_Application->GetRenderer()->GetSourceResolution().Y -
                 m_CurrentSplash->GetSize().Y) / 2.f);

            m_CurrentSplashTime = sf::core::g_TimeManager::Instance().GetTime();
            break;
        }
    }
}

void CBranch::LoadSplash(const char *_id)
{
	sf::graphics::CTexture *s = sf::core::g_TextureManager::Instance().GetItem(_id);
	if(s && s->GetWidth() > 2 && s->GetHeight() > 2)
	{
		m_Splashes.push_back(SplashInfo());
		m_Splashes.back().m_Image  = new sf::graphics::CImage(s);
		m_Splashes.back().ShowTime = 1000;
		m_Splashes.back().Time = 4000;
	}
}

void CBranch::OnAdd(sf::gui::CBaseWidget* _parent)
{
    sf::gui::CWidget::OnAdd(_parent);
    if (_parent)
    {
        SetOffset(0.f, 0.f);
        SetSize(_parent->GetSize().X, _parent->GetSize().Y);
    }
}

bool CBranch::OnMouseDown(const sf::misc::IntVector& _pos, MouseButtons _button, int _keyboard_state, bool _broadcast)
{
	if(CWidget::OnMouseDown(_pos, _button, _keyboard_state, _broadcast))
		return true;
		
	if(!m_Splashes.empty())
	{
		m_Splashes.pop_front();
		ShowSplash();
	}
	else
	{
		m_CurrentSplash->SetColor(0);
		m_SplashEnded = true;
	}
	
	return true;
}
	
} // namespace game
