#include "ResourcesDownloader.h"

#include <sf/core/android/jni.h>
#include <sf/diag/log.h>

namespace game
{
// jni vars
static jclass g_ResourcesDownloaderClass = 0;
static jmethodID g_MethodDownloadFile = 0;
static jmethodID g_MethodIsDownloading = 0;
static jmethodID g_MethodIsError = 0;
static jmethodID g_MethodStopDownload = 0;
static jmethodID g_MethodResetDownload = 0;
static jmethodID g_MethodGetContentLength = 0;
static jmethodID g_MethodGetDownloadedLength = 0;
static jmethodID g_MethodGetResourcesSize = 0;

// single instance of CResourcesDownloader
static CResourcesDownloader* g_Instance = 0;

CResourcesDownloader& CResourcesDownloader::Instance()
{
    if (!g_Instance) g_Instance = new CResourcesDownloader;
    return *g_Instance;
}

CResourcesDownloader::~CResourcesDownloader()
{
}

bool CResourcesDownloader::DownloadFile(const char* _source_link)
{
    jstring link = sf::core::g_JavaEnv->NewStringUTF(_source_link);
    bool result = sf::core::g_JavaEnv->CallStaticBooleanMethod(g_ResourcesDownloaderClass, g_MethodDownloadFile, link);
    sf::core::g_JavaEnv->DeleteLocalRef(link);
    
    return result;
}

bool CResourcesDownloader::IsDownloading()
{
    return sf::core::g_JavaEnv->CallStaticBooleanMethod(g_ResourcesDownloaderClass, g_MethodIsDownloading);
}

bool CResourcesDownloader::IsError()
{
	return sf::core::g_JavaEnv->CallStaticBooleanMethod(g_ResourcesDownloaderClass, g_MethodIsError);
}

void CResourcesDownloader::StopDownload()
{
    return sf::core::g_JavaEnv->CallStaticVoidMethod(g_ResourcesDownloaderClass, g_MethodStopDownload);
}

void CResourcesDownloader::ResetDownload(const char* _source_link)
{
    jstring link = sf::core::g_JavaEnv->NewStringUTF(_source_link);
    sf::core::g_JavaEnv->CallStaticBooleanMethod(g_ResourcesDownloaderClass, g_MethodResetDownload, link);
    sf::core::g_JavaEnv->DeleteLocalRef(link);
}
    
int CResourcesDownloader::GetContentLength()
{
    return sf::core::g_JavaEnv->CallStaticIntMethod(g_ResourcesDownloaderClass, g_MethodGetContentLength);
}

int CResourcesDownloader::GetDownloadedLength()
{
    return sf::core::g_JavaEnv->CallStaticIntMethod(g_ResourcesDownloaderClass, g_MethodGetDownloadedLength);
}

int CResourcesDownloader::GetResourcesSize(const char* _source_link)
{   
    jstring link = sf::core::g_JavaEnv->NewStringUTF(_source_link);
    int result = sf::core::g_JavaEnv->CallStaticIntMethod(g_ResourcesDownloaderClass, g_MethodGetResourcesSize, link);
    sf::core::g_JavaEnv->DeleteLocalRef(link);
    
    return result;
}

CResourcesDownloader::CResourcesDownloader()
{
    if (!g_ResourcesDownloaderClass)
    {
        g_ResourcesDownloaderClass = sf::core::JNIGetClass("com/stargaze/network/ResourcesDownloader");
        
        g_MethodDownloadFile = sf::core::JNIGetStaticMethod(g_ResourcesDownloaderClass,
            "downloadFile", "(Ljava/lang/String;)Z");

        g_MethodIsDownloading = sf::core::JNIGetStaticMethod(g_ResourcesDownloaderClass,
            "isDownloading", "()Z");

        g_MethodIsError = sf::core::JNIGetStaticMethod(g_ResourcesDownloaderClass,
            "isError", "()Z");

        g_MethodStopDownload = sf::core::JNIGetStaticMethod(g_ResourcesDownloaderClass,
            "stopDownload", "()V");

        g_MethodResetDownload = sf::core::JNIGetStaticMethod(g_ResourcesDownloaderClass,
            "resetDownload", "(Ljava/lang/String;)V");

        g_MethodGetContentLength = sf::core::JNIGetStaticMethod(g_ResourcesDownloaderClass,
            "getContentLength", "()I");

        g_MethodGetDownloadedLength = sf::core::JNIGetStaticMethod(g_ResourcesDownloaderClass,
            "getDownloadedLength", "()I");

        g_MethodGetResourcesSize = sf::core::JNIGetStaticMethod(g_ResourcesDownloaderClass,
            "getResourcesSize", "(Ljava/lang/String;)I");
    }
}
    
} // game
