#include "../../sf.h"

#include "jni.h"

#include "../../diag/log.h"
#include "../../diag/assert.h"

namespace sf
{
namespace core
{

JavaVM* g_JavaVM = 0;
JNIEnv* g_JavaEnv = 0;
jobject g_GameThread = 0;

extern "C" jint JNI_OnLoad(JavaVM* _vm, void* _)
{
    g_JavaVM = _vm;
    return JNI_VERSION_1_4;
}

jclass JNIGetClass(const char* _class)
{
   jclass cls = g_JavaEnv->FindClass(_class);
    if (!cls) 
    {
        SFLOGA("sf", sf::diag::LogError, "Class not found: '%s'", _class);
        SFASSERT(cls);
    }
    
    return cls;
}

jmethodID JNIGetObjectMethod(jclass _class, const char* _method, const char* _signature)
{
    jmethodID method = g_JavaEnv->GetMethodID(_class, _method, _signature);
    if (!method)
    {
        SFLOGA("sf", sf::diag::LogError, "Method not found: '%s' '%s'", _method, _signature);
        SFASSERT(method);
    }
    
    return method;
}

jmethodID JNIGetStaticMethod(jclass _class, const char* _method, const char* _signature)
{
    jmethodID method = g_JavaEnv->GetStaticMethodID(_class, _method, _signature);
    if (!method)
    {
        SFLOGA("sf", sf::diag::LogError, "Static method not found: '%s' '%s'", _method, _signature);
        SFASSERT(method);
    }
    
    return method;
}

jfieldID JNIGetStaticField(jclass _class, const char* _field, const char* _type)
{
    jfieldID field = g_JavaEnv->GetStaticFieldID(_class, _field, _type);
    if (!field)
    {
        SFLOGA("sf", sf::diag::LogError, "Field not found: '%s' '%s'", _field, _type);
        SFASSERT(field);
    }
    
    return field;
}

}
}