#include <sf/lua/lua.h>
#include <sf/lua/luabindhelpers.h>
#include <sf/lua/luabindings.h>
#include <sf/misc/StringFormat.h>

#ifdef __SF_ANDROID
#include <sys/vfs.h>
#include "InAppBilling.h"
#include "ResourcesDownloader.h"
#else
#include "windows/InAppBilling.h"
#include "windows/ResourcesDownloader.h"
#undef GetFreeSpace
#endif

#include "FlurryAgent.h"

namespace game
{
           
static int GetFreeSpace()
{
#ifdef __SF_ANDROID
	struct statfs s;
	if(statfs(".", &s) == -1)
		return -1;
	uint64_t size = s.f_bfree * s.f_bsize;
	return size / (1024 * 1024);
#else
	return -1;
#endif
}

static int GetTotalSpace()
{
#ifdef __SF_ANDROID
	struct statfs s;
	if(statfs(".", &s) == -1)
		return -1;
	uint64_t size = s.f_blocks * s.f_bsize;
	return size / (1024 * 1024);
#else
	return -1;
#endif
}

SF_WSTRING StringFormat(const SF_WSTRING& _format, int _value_1)
{
	const int size(1024);
	wchar_t buffer[size];
        sf::misc::StringFormatW(buffer, size, _format.c_str(), _value_1);
	return buffer;
}

SF_WSTRING StringFormat(const SF_WSTRING& _format, int _value_1, int _value_2, int _value_3)
{
	const int size(1024);
	wchar_t buffer[size];
        sf::misc::StringFormatW(buffer, size, _format.c_str(), _value_1, _value_2, _value_3);
	return buffer;
}

void LuaBindLibgame(const char * _prefix, sf::lua::CLua & _lua)
{
	using namespace mluabind;
	_lua.Insert
	(
		Declare(_prefix)

            + LUA_FUNCTION(GetFreeSpace)
			+ LUA_FUNCTION(GetTotalSpace)

            + LUA_FUNCTION_OVERLOAD(StringFormat, SF_WSTRING (*)(const SF_WSTRING&, int))
			+ LUA_FUNCTION_OVERLOAD(StringFormat, SF_WSTRING (*)(const SF_WSTRING&, int, int, int)) 

			+ LUA_CLASS(CResourcesDownloader)
                .LUA_CLASS_METHOD(CResourcesDownloader, DownloadFile)
                .LUA_CLASS_METHOD(CResourcesDownloader, IsDownloading)
                .LUA_CLASS_METHOD(CResourcesDownloader, IsError)
                .LUA_CLASS_METHOD(CResourcesDownloader, StopDownload)
                .LUA_CLASS_METHOD(CResourcesDownloader, ResetDownload)
                .LUA_CLASS_METHOD(CResourcesDownloader, GetContentLength)
                .LUA_CLASS_METHOD(CResourcesDownloader, GetDownloadedLength)
                .LUA_CLASS_METHOD(CResourcesDownloader, GetResourcesSize)
                                                        
            + (Declare("CResourcesDownloader")
                + Function("Instance", &CResourcesDownloader::Instance)
            )

			+ LUA_CLASS(CInAppBilling)
                .LUA_CLASS_ENUM(CInAppBilling, StateAvailable)
                .LUA_CLASS_ENUM(CInAppBilling, StatePending)
                .LUA_CLASS_ENUM(CInAppBilling, StatePurchased)
                .LUA_CLASS_METHOD(CInAppBilling, IsServiceAvailable)
                .LUA_CLASS_METHOD(CInAppBilling, IsServiceConnected)
                .LUA_CLASS_METHOD(CInAppBilling, IsBillingSupported)
                .LUA_CLASS_METHOD(CInAppBilling, IsOnline)
                .LUA_CLASS_METHOD(CInAppBilling, InitiatePurchase)
                .LUA_CLASS_METHOD(CInAppBilling, GetItemPurchaseState)
                .LUA_CLASS_METHOD(CInAppBilling, PurchaseDelivered)
                .LUA_CLASS_METHOD(CInAppBilling, IsPurchaseDelivered)
            + (Declare("CInAppBilling")
                + Function("Instance", &CInAppBilling::Instance)
            )
			+ LUA_CLASS(CFlurryAgent)
                .LUA_CLASS_METHOD(CFlurryAgent, OnMoreGames)
                .LUA_CLASS_METHOD(CFlurryAgent, OnNewsletter)
                .LUA_CLASS_METHOD(CFlurryAgent, OnRateIt)
                .LUA_CLASS_METHOD(CFlurryAgent, OnRemindMeLater)
                .LUA_CLASS_METHOD(CFlurryAgent, OnNoThanks)
                .LUA_CLASS_METHOD(CFlurryAgent, OnStartPurchase)
                .LUA_CLASS_METHOD(CFlurryAgent, OnPurchaseCompleted)
                .LUA_CLASS_METHOD(CFlurryAgent, OnPurchaseScreenClose)
            + (Declare("CFlurryAgent")
                + Function("Instance", &CFlurryAgent::Instance)
            )
	);
}

}
