#include <sf/sf.h>

#include <sf/lua/lua.h>
#include <sf/lua/luabindhelpers.h>
#include <sf/core/application.h>
#include <sf/core/texturemanager.h>
#include <sf/core/audiomanager.h>
#include <sf/core/fontmanager.h>
#include <sf/core/timer.h>
#include <sf/core/settings.h>
#include <sf/graphics/surface.h>
#include <sf/gui/basewindow.h>

namespace game
{
namespace lua
{

enum SF_Platform
{
	SF_Unknown = 0,
	SF_Windows = 1,
	SF_MacOSX = 2,
	SF_iPhone = 3,
	SF_Android = 4
};
	
static const SF_Platform g_SFPlatform = 
#if defined(__SF_WINDOWS)
	SF_Windows
#elif defined(__SF_MAC)
	SF_MacOSX
#elif defined(__SF_IPHONE)
	SF_iPhone
#elif defined(__SF_ANDROID)
	SF_Android
#else
	SF_Unknown
#endif
;
	
void BindCore(sf::lua::CLua& _lua)
{
	using namespace mluabind;
	using namespace sf;
	using namespace sf::core;

	_lua.StartBinding();

	_lua.Insert
	(
		Declare("sf")
			+ (Declare("core")
				+ LUA_ENUM(SF_Unknown)
				+ LUA_ENUM(SF_Windows)
				+ LUA_ENUM(SF_MacOSX)
				+ LUA_ENUM(SF_iPhone)
				+ LUA_ENUM(SF_Android)			   

				+ LUA_CLASS(CApplication)
			   
					.LUA_CLASS_ENUM(CApplication, DebugNone)
					.LUA_CLASS_ENUM(CApplication, DebugCheats)
					.LUA_CLASS_ENUM(CApplication, DebugLight)
					.LUA_CLASS_ENUM(CApplication, DebugHeavy)

					.LUA_CLASS_METHOD(CApplication, Load)
					.LUA_CLASS_METHOD(CApplication, SetMainWindow)
					.LUA_CLASS_METHOD(CApplication, GetMainWindow)
					.LUA_CLASS_METHOD(CApplication, GetTopWindow)
					.LUA_CLASS_METHOD(CApplication, ChangeScreenMode)
					.LUA_CLASS_METHOD(CApplication, GetAppDataFolder)
					.Method("Shutdown", &CApplication::SoftShutdown)
                    .LUA_CLASS_METHOD(CApplication, Stop)
//					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetSettings, const CSettings& (CApplication::*)() const)
					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetSettings, CSettings& (CApplication::*)())
//					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetResourceDeclarations, const CResourceDeclarations& (CApplication::*)() const)
					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetResourceDeclarations, CResourceDeclarations& (CApplication::*)())
					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetTimeManager, CTimeManager& (CApplication::*)())
//					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetRenderer, const sf::graphics::CRenderer* (CApplication::*)() const)
					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetRenderer, sf::graphics::CRenderer* (CApplication::*)())
					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetTextureManager, CTextureManager& (CApplication::*)())
					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetFontManager, CFontManager& (CApplication::*)())
					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetAudioManager, core::CAudioManager& (CApplication::*)())
//                    .LUA_CLASS_METHOD_OVERLOAD(CApplication, GetWidgetManager, const sf::gui::CWidgetManager& (CApplication::*)() const)
					.LUA_CLASS_METHOD_OVERLOAD(CApplication, GetWidgetManager, sf::gui::CWidgetManager& (CApplication::*)())

					.LUA_CLASS_METHOD(CApplication, GetDebugLevel)
#ifndef __SF_IPHONE			   
					.LUA_CLASS_METHOD(CApplication, SetDebugLevel)
                    .LUA_CLASS_METHOD(CApplication, LimitFPS)
#endif
			   
				+ LUA_CLASS(CTextureManager)
					.LUA_CLASS_METHOD_OVERLOAD(CTextureManager, GetItem, const sf::graphics::CTexture* (CTextureManager::*)(const char*) const)
//					.LUA_CLASS_METHOD_OVERLOAD(CTextureManager, GetItem, sf::graphics::CTexture* (CTextureManager::*)(const char*))
					.LUA_CLASS_METHOD_OVERLOAD(CTextureManager, GetItemId, const char* (CTextureManager::*)(const sf::graphics::CTexture&) const)
//					.LUA_CLASS_METHOD_OVERLOAD(CTextureManager, GetItemId, const char* (CTextureManager::*)(const sf::graphics::CTexture&))
                    .LUA_CLASS_METHOD(CTextureManager, UnloadAllOnDemandSurfaces)
					.LUA_CLASS_METHOD(CTextureManager, PreloadSurfacesByPrefix)
//                    .LUA_CLASS_METHOD(CTextureManager, UnloadTextureDataDeferred)

				+ LUA_CLASS(CFontManager)
					.LUA_CLASS_METHOD_OVERLOAD(CFontManager, GetItem, const sf::graphics::CFont* (CFontManager::*)(const char*) const)
//					.LUA_CLASS_METHOD_OVERLOAD(CFontManager, GetItem, sf::graphics::CFont* (CFontManager::*)(const char*))
					.LUA_CLASS_METHOD_OVERLOAD(CFontManager, GetItemId, const char* (CFontManager::*)(const sf::graphics::CFont&) const)
//					.LUA_CLASS_METHOD_OVERLOAD(CFontManager, GetItemId, const char* (CFontManager::*)(const sf::graphics::CFont&))

				+ LUA_CLASS(CAudioManager)
					.LUA_CLASS_METHOD(CAudioManager, PauseAll)	
					.LUA_CLASS_METHOD(CAudioManager, GetGlobalVolume)	
					.LUA_CLASS_METHOD(CAudioManager, SetGlobalVolume)
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, SetVolume, bool (CAudioManager::*)(size_t , float , float , float))
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, SetVolume, bool (CAudioManager::*)(const IdString& , float , float , float))
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, GetVolume, float (CAudioManager::*)(size_t) const)
                    .LUA_CLASS_METHOD_OVERLOAD(CAudioManager, GetVolume, float (CAudioManager::*)(const IdString&) const)
                    .LUA_CLASS_METHOD(CAudioManager, GetMaxVolume)
					.LUA_CLASS_METHOD(CAudioManager, GetPosition)
					.LUA_CLASS_METHOD(CAudioManager, SetPosition)
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, GetPriority, int (CAudioManager::*)(size_t) const)
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, GetRepeatCount, int (CAudioManager::*)(size_t) const)
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, GetLength, double (CAudioManager::*)(const IdString &) const)
                    .LUA_CLASS_METHOD_OVERLOAD(CAudioManager, GetLength, double (CAudioManager::*)(size_t) const)
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, Play, size_t (CAudioManager::*)(const IdString &, float , float , int , int , float))
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, StopAll, void (CAudioManager::*)(float))
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, Stop, bool (CAudioManager::*)(size_t, float))
                    .LUA_CLASS_METHOD_OVERLOAD(CAudioManager, Stop, bool (CAudioManager::*)(const IdString&, float))
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, IsSound, bool (CAudioManager::*)(const IdString &) const)
					.LUA_CLASS_METHOD_OVERLOAD(CAudioManager, IsSound, bool (CAudioManager::*)(size_t) const)
					.LUA_CLASS_METHOD(CAudioManager, SetGroupVolume)
					.LUA_CLASS_METHOD(CAudioManager, GetGroupVolume)
					.LUA_CLASS_METHOD(CAudioManager, StopGroup)
					.LUA_CLASS_METHOD(CAudioManager, UnloadAllOnDemandSounds)

				+ LUA_CLASS(CSettingsGroup)
					.Constructor<const IdString&>()
					.LUA_CLASS_METHOD(CSettingsGroup, GetId)
					.LUA_CLASS_METHOD(CSettingsGroup, AddChild)
					.LUA_CLASS_METHOD(CSettingsGroup, RemoveChild)
					.LUA_CLASS_METHOD(CSettingsGroup, EnumChildren)
					.LUA_CLASS_METHOD(CSettingsGroup, GetFirstChild)
					.LUA_CLASS_METHOD(CSettingsGroup, GetFirstChildRef)
					.LUA_CLASS_METHOD(CSettingsGroup, GetChild)
					.LUA_CLASS_METHOD(CSettingsGroup, GetChildRef)
					.LUA_CLASS_METHOD(CSettingsGroup, GetChildByAttribute)
					.LUA_CLASS_METHOD(CSettingsGroup, GetChildByAttributeRef)
					.LUA_CLASS_METHOD(CSettingsGroup, GetNamedChild)
					.LUA_CLASS_METHOD(CSettingsGroup, GetNamedChildRef)
					.LUA_CLASS_METHOD(CSettingsGroup, GetNextSibling)
					.LUA_CLASS_METHOD(CSettingsGroup, GetNextSiblingRef)
                    .LUA_CLASS_METHOD_OVERLOAD(CSettingsGroup, AddValue, void (CSettingsGroup::*)(const IdString&, const std::string&))
                    .LUA_CLASS_METHOD_OVERLOAD(CSettingsGroup, AddValue, void (CSettingsGroup::*)(const char*, const char*))
					.LUA_CLASS_METHOD(CSettingsGroup, IsValue)
					.Method("GetValue", (const std::string& (CSettingsGroup::*)(const IdString&) const) &CSettingsGroup::GetValue)
			   
					.LUA_CLASS_METHOD(CSettingsGroup, EnumAttributes)
					.LUA_CLASS_METHOD(CSettingsGroup, GetAttribute)
					.LUA_CLASS_METHOD(CSettingsGroup, CopyTo)
				
			   + mluabind::Class<CSettingsGroup::SettingsGroupPtrT>("SettingsGroupPtrT")
					.Constructor<CSettingsGroup*>(Adopt(1))//, adopt(_1)
					.LUA_CLASS_METHOD(CSettingsGroup::SettingsGroupPtrT, get)

				+ LUA_CLASS(CSettings)
					.BaseNoDynCast<CSettingsGroup>()
					.Constructor()
					.LUA_CLASS_METHOD(CSettings, LoadXml)
					.LUA_CLASS_METHOD(CSettings, LoadXmlDirect)
				
				+ LUA_CLASS(CTimer)
					.LUA_CLASS_METHOD(CTimer, SetK)
					.LUA_CLASS_METHOD(CTimer, GetK)
					.LUA_CLASS_METHOD(CTimer, GetTime)
					.LUA_CLASS_METHOD(CTimer, SetTime)
					.LUA_CLASS_METHOD(CTimer, GetFrameDelta)
					.LUA_CLASS_METHOD(CTimer, Pause)
					.LUA_CLASS_METHOD(CTimer, IsPaused)
					.LUA_CLASS_METHOD(CTimer, Destroy)

				+ LUA_CLASS(CGroupTimer)
					.LUA_CLASS_METHOD(CGroupTimer, GetId)
					.LUA_CLASS_METHOD(CGroupTimer, AttachTimer)
					.LUA_CLASS_METHOD(CGroupTimer, AttachGroupTimer)
					.LUA_CLASS_METHOD(CGroupTimer, GetChildGroupTimer)					
					.LUA_CLASS_METHOD(CGroupTimer, SetK)
					.LUA_CLASS_METHOD(CGroupTimer, GetK)
					.LUA_CLASS_METHOD(CGroupTimer, GetTime)
					.LUA_CLASS_METHOD(CGroupTimer, SetTime)
					.LUA_CLASS_METHOD(CGroupTimer, GetFrameDelta)
					.LUA_CLASS_METHOD(CGroupTimer, Pause)
					.LUA_CLASS_METHOD(CGroupTimer, IsPaused)
					.LUA_CLASS_METHOD(CGroupTimer, Destroy)


				+ LUA_CLASS(CTimerWrapper)
					.Constructor<CTimer*>()
					.Constructor<const CTimerWrapper&>()
					.LUA_CLASS_METHOD_OVERLOAD(CTimerWrapper, Get, CTimer* (CTimerWrapper::*)())
					.LUA_CLASS_METHOD_OVERLOAD(CTimerWrapper, Get, const CTimer* (CTimerWrapper::*)() const)

				+ LUA_CLASS(CGroupTimerWrapper)
					.Constructor<CGroupTimer*>()
					.Constructor<const CGroupTimerWrapper&>()
					.LUA_CLASS_METHOD_OVERLOAD(CGroupTimerWrapper, Get, CGroupTimer* (CGroupTimerWrapper::*)())
					.LUA_CLASS_METHOD_OVERLOAD(CGroupTimerWrapper, Get, const CGroupTimer* (CGroupTimerWrapper::*)() const)

				+ LUA_CLASS(CTimeManager)
					.LUA_CLASS_METHOD(CTimeManager, Pause)
					.LUA_CLASS_METHOD(CTimeManager, GetTime)
                    .LUA_CLASS_METHOD(CTimeManager, GetAppTime)
					.LUA_CLASS_METHOD(CTimeManager, AttachTimer)
					.LUA_CLASS_METHOD(CTimeManager, AttachGroupTimer)
					.LUA_CLASS_METHOD(CTimeManager, GetTimer)
					.LUA_CLASS_METHOD(CTimeManager, RemoveTimer)
					.LUA_CLASS_METHOD(CTimeManager, RemoveGroupTimer)

				+ LUA_CONSTANT(g_Application)
				+ LUA_CONSTANT(g_SFPlatform)

			)
	);

	_lua.FinishBinding();
}

} //lua
} //game