#include <sf/sf.h>
#include <sf/lua/lua.h>
#include <sf/lua/LuaBindHelpers.h>
#include <sf/core/Settings.h>
#include <sf/graphics/color.h>
#include <sf/graphics/font.h>
#include <sf/graphics/renderer.h>
#include <sf/graphics/texture.h>
#include <sf/graphics/surface.h>
#include <sf/graphics/particle.h>
#include <sf/graphics/videoplayer.h>

namespace game
{
namespace lua
{

static unsigned int TextureGetPixelConst(const sf::graphics::CTexture* _this, int _x, int _y, size_t _frame)
{
	const unsigned int* p = (*_this)(_x, _y, _frame);
	return p ? *p : 0;
}

static bool TextureSurfaceIsBuffer(const sf::graphics::CTexture* _this)
{
	return (_this->GetSurface()->GetBuffer()!=0);
}

//   mluabind      9 
//   ,   
static void CRendererRenderText(const sf::graphics::CRenderer* _renderer, const sf::graphics::CFont& _font, const SF_WSTRING& _string,
					sf::misc::IntRect& _dst_box, unsigned _draw_flags, unsigned _count,
					const sf::graphics::Color& _color, const sf::graphics::Color& _bk_color)
{
    _renderer->RenderText(_font, _string, _dst_box, _draw_flags, _count, _color, _bk_color);
}

void BindGraphics(sf::lua::CLua& _lua)
{
	using namespace mluabind;
	using namespace sf;
	using namespace sf::graphics;

	_lua.StartBinding();

	_lua.Insert
	(
		Declare("sf")
			+ (Declare("graphics")
				+ LUA_CONSTANT(BlendNormal)
				+ LUA_CONSTANT(BlendAdditive)

				+ LUA_CLASS(Color)
					.Constructor()
					.Constructor<unsigned long>()
					.Constructor<int, int, int>()
					.Constructor<int, int, int, int>()
					.Constructor<const Color&>()
					.Operator(Mul, static_cast<Color (Color::*)(const Color&) const>(&Color::operator*))
					.Operator(Mul, static_cast<Color (Color::*)(float) const>(&Color::operator*))
					.Operator(Sub, static_cast<Color (Color::*)(const Color&) const>(&Color::operator-))
					.Operator(Add, static_cast<Color (Color::*)(const Color&) const>(&Color::operator+))
					.LUA_CLASS_MEMBER(Color, Red)
					.LUA_CLASS_MEMBER(Color, Green)
					.LUA_CLASS_MEMBER(Color, Blue)
					.LUA_CLASS_MEMBER(Color, Alpha)
					.LUA_CLASS_METHOD(Color, Clamp)
					.Operator(self==self)

				+ LUA_CLASS(CFont)
					.LUA_CLASS_METHOD(CFont, GetStringWidth)
					.LUA_CLASS_MEMBER(CFont, PixHeight)
//					.LUA_CLASS_MEMBER(CFont, SpaceWidth)
//					.LUA_CLASS_MEMBER(CFont, LineSpacing)
//					.LUA_CLASS_MEMBER(CFont, Texture)

				+ LUA_CLASS(CRenderer)
//					.LUA_CLASS_METHOD(CRenderer, InitWindow)
//					.LUA_CLASS_METHOD(CRenderer, UpdateWindowInfo)
//					.LUA_CLASS_METHOD(CRenderer, SetSourceResolution)
					.LUA_CLASS_METHOD(CRenderer, GetSourceResolution)
					.LUA_CLASS_METHOD(CRenderer, RenderLine)
					.LUA_CLASS_METHOD(CRenderer, RenderRect)
					.LUA_CLASS_METHOD(CRenderer, RenderFilledPoly4)
					.LUA_CLASS_METHOD_OVERLOAD(CRenderer, RenderString, void (CRenderer::*)(const CFont*, const SF_WSTRING&, float, float, int, int, float, const Color&, const Color&) const)
                    .Method("RenderText", &CRendererRenderText)
					.LUA_CLASS_METHOD_OVERLOAD(CRenderer, RenderTexture, void (CRenderer::*)(const CTexture*, float, float, size_t, const Color&) const)
					.LUA_CLASS_METHOD_OVERLOAD(CRenderer, RenderTexture, void (CRenderer::*)(const CTexture*, float, float, const misc::ShortRect&, const Color&) const)
					.LUA_CLASS_METHOD_OVERLOAD(CRenderer, RenderTexture, void (CRenderer::*)(const CTexture*, const misc::FloatRect&, size_t, const Color&) const)
					.LUA_CLASS_METHOD_OVERLOAD(CRenderer, RenderTexture, void (CRenderer::*)(const CTexture*, const misc::FloatRect&, const misc::ShortRect&, const Color&) const)
					.LUA_CLASS_METHOD_OVERLOAD(CRenderer, RenderTexture, void (CRenderer::*)(const CTexture*, const misc::Poly4&, size_t, const Color&) const)
					.LUA_CLASS_METHOD_OVERLOAD(CRenderer, RenderTexture, void (CRenderer::*)(const CTexture*, const misc::Poly4&, const misc::ShortRect&, const Color&) const)
					.LUA_CLASS_METHOD_OVERLOAD(CRenderer, RenderTextureBox, void (CRenderer::*)(const CTexture*, const misc::FloatRect&, size_t, const Color&) const)
					.LUA_CLASS_METHOD_OVERLOAD(CRenderer, RenderTextureBox, void (CRenderer::*)(const CTexture*, const misc::FloatRect&, const misc::ShortRect&, const Color&) const)
//					.LUA_CLASS_METHOD(CRenderer, BeginScene)
//					.LUA_CLASS_METHOD(CRenderer, EndScene)
					.LUA_CLASS_METHOD(CRenderer, SetBlendMode)
					.LUA_CLASS_METHOD(CRenderer, SetColor)
					.LUA_CLASS_METHOD(CRenderer, GetColor)
					.LUA_CLASS_METHOD(CRenderer, ApplyMatrix)
					.LUA_CLASS_METHOD(CRenderer, GetMatrix)
					.LUA_CLASS_METHOD(CRenderer, SetClipPoly)
					.LUA_CLASS_METHOD(CRenderer, PushState)
					.LUA_CLASS_METHOD(CRenderer, PopState)
//					.LUA_CLASS_METHOD(CRenderer, GetWidth)
//					.LUA_CLASS_METHOD(CRenderer, GetHeight)
//					.LUA_CLASS_METHOD(CRenderer, Is32BPP)
//					.LUA_CLASS_METHOD(CRenderer, IsFullscreen)
//					.LUA_CLASS_METHOD(CRenderer, GetActiveRect)
//					.LUA_CLASS_METHOD(CRenderer, GetScale)
//					.LUA_CLASS_METHOD(CRenderer, GetAvailableResolutions)
//				+ (Declare("CRenderer")
//					+ BindList<CRenderer::ResolutionsT>("ResolutionsT")
//				)
//				+ LUA_CLASS(Resolution)
//					.LUA_CLASS_MEMBER(Resolution, Width)
//					.LUA_CLASS_MEMBER(Resolution, Height)
//					.LUA_CLASS_MEMBER(Resolution, Bpp32)

				+ LUA_CLASS(CSurface)
//					.LUA_CLASS_METHOD(CSurface, IsValid)
//					.LUA_CLASS_METHOD(CSurface, GetWidth)
//					.LUA_CLASS_METHOD(CSurface, GetHeight)

				+ LUA_CLASS(CTexture)
					.Constructor()
					.LUA_CLASS_METHOD_OVERLOAD(CTexture, GetSurface, const CSurface* (CTexture::*) () const)
					.LUA_CLASS_METHOD_OVERLOAD(CTexture, GetSurface, CSurface* (CTexture::*) ())
					.LUA_CLASS_METHOD(CTexture, GetSurfaceRect)
//					.LUA_CLASS_METHOD(CTexture, CreateBuffer)
//					.LUA_CLASS_METHOD(CTexture, LoadBuffer)
					.LUA_CLASS_METHOD(CTexture, DropBuffer)
//					.Method("Pixel", &TextureGetPixelConst)
					.Method("SurfaceIsBuffer", &TextureSurfaceIsBuffer)
//					.LUA_CLASS_METHOD(CTexture, CommitBits)
					.LUA_CLASS_METHOD(CTexture, IsAnimated)
					.LUA_CLASS_METHOD(CTexture, EnumFrames)
					.LUA_CLASS_METHOD(CTexture, GetFrameSurfaceRect)
					.LUA_CLASS_METHOD(CTexture, GetFrameByTime)
					.LUA_CLASS_METHOD(CTexture, GetTimeByFrame)
					.LUA_CLASS_METHOD(CTexture, GetWidth)
					.LUA_CLASS_METHOD(CTexture, GetHeight)
					.LUA_CLASS_METHOD(CTexture, GetAnimTime)

				+ LUA_CLASS(CParticleSystem)
					.Constructor<sf::core::CTimer*>()
					.Constructor<CParticleSystem&>()
					.LUA_CLASS_ENUM(CParticleSystem, IsStopEmit)
					.LUA_CLASS_ENUM(CParticleSystem, IsContinuous)
					.LUA_CLASS_METHOD(CParticleSystem, LoadById)
					.LUA_CLASS_METHOD(CParticleSystem, Reset)
					.LUA_CLASS_METHOD(CParticleSystem, Update)
					.LUA_CLASS_METHOD(CParticleSystem, Render)
					.LUA_CLASS_METHOD(CParticleSystem, GetLifeParticlesCount)
					.LUA_CLASS_METHOD(CParticleSystem, DropOldPos)
					.LUA_CLASS_METHOD(CParticleSystem, IsDead)
					.LUA_CLASS_METHOD(CParticleSystem, GetFlags)
					.LUA_CLASS_METHOD(CParticleSystem, SetFlags)
                    .LUA_CLASS_METHOD(CParticleSystem, GetEmission)
                    .LUA_CLASS_METHOD(CParticleSystem, SetEmission)
                    .LUA_CLASS_METHOD(CParticleSystem, GetEmiterSize)
                    .LUA_CLASS_METHOD(CParticleSystem, SetTimer)
					.LUA_CLASS_METHOD_OVERLOAD(CParticleSystem, SetEmiterSize, void (CParticleSystem::*)(const sf::misc::FloatVector&))
					.LUA_CLASS_METHOD_OVERLOAD(CParticleSystem, SetLocation, void (CParticleSystem::*)(float , float) )

			   + LUA_CLASS(CImage)
					.LUA_CLASS_ENUM(CImage, FlagBoxImage)
					.LUA_CLASS_ENUM(CImage, FlagIntDest)
					.Constructor<const TextureIdString&>()
					.Constructor<const CTexture*>()
					.LUA_CLASS_METHOD(CImage, Init)
					.LUA_CLASS_METHOD_OVERLOAD(CImage, SetTexture, void (CImage::*)(const CTexture*, bool))
					.LUA_CLASS_METHOD_OVERLOAD(CImage, SetTexture, void (CImage::*)(const TextureIdString&, bool))
					.LUA_CLASS_METHOD(CImage, GetTexture)
					.LUA_CLASS_METHOD(CImage, GetTextureSize)
					.LUA_CLASS_METHOD_OVERLOAD(CImage, Draw, void (CImage::*)(CRenderer*) const)
                    .LUA_CLASS_METHOD_OVERLOAD(CImage, Draw, void (CImage::*)(CRenderer*, float, float) const)
					.LUA_CLASS_METHOD_OVERLOAD(CImage, Draw, void (CImage::*)(CRenderer*, const misc::FloatRect&) const)
					.LUA_CLASS_METHOD_OVERLOAD(CImage, DrawTransformed, void (CImage::*)(CRenderer*, const misc::FloatMatrix&)const)
					.LUA_CLASS_METHOD_OVERLOAD(CImage, DrawTransformed, 
						void (CImage::*)(CRenderer*, const misc::FloatRect&, const misc::FloatMatrix&)const)
					.LUA_CLASS_METHOD(CImage, SetRect)
					.LUA_CLASS_METHOD(CImage, GetRect)
					.LUA_CLASS_METHOD(CImage, SetSourceRect)
					.LUA_CLASS_METHOD(CImage, GetSourceRect)
					.LUA_CLASS_METHOD(CImage, SetTransformationMatrix)
					.LUA_CLASS_METHOD(CImage, GetTransformationMatrix)
					.LUA_CLASS_METHOD(CImage, SetScale)
					.LUA_CLASS_METHOD(CImage, GetScale)
					.LUA_CLASS_METHOD(CImage, SetColor)
					.LUA_CLASS_METHOD(CImage, GetColor)
					.LUA_CLASS_METHOD(CImage, SetFlags)
					.LUA_CLASS_METHOD(CImage, GetFlags)
					.LUA_CLASS_METHOD(CImage, IsAnimated)
					.LUA_CLASS_METHOD(CImage, GetFrameCount)
					.LUA_CLASS_METHOD(CImage, PauseAnimation)
					.LUA_CLASS_METHOD(CImage, SetAnimationK)
					.LUA_CLASS_METHOD(CImage, GetAnimationK)
					.LUA_CLASS_METHOD(CImage, SetFrame)
					.LUA_CLASS_METHOD(CImage, SetTime)
					.LUA_CLASS_METHOD(CImage, GetTime)
#if defined(__SF_WINDOWS) || defined(__SF_MAC)
                + LUA_CLASS(CVideoPlayer)
                    .Constructor<core::CGroupTimer*>()
                    .LUA_CLASS_METHOD(CVideoPlayer, LoadFromFile)
#elif defined(__SF_IPHONE)
			   + LUA_CLASS(CVideoPlayer)
					.Constructor()
					.LUA_CLASS_METHOD(CVideoPlayer, PlayFromFile)
#endif
			)
	);

	_lua.FinishBinding();
}

} //lua
} //game
