#include <sf/sf.h>
#include <sf/lua/lua.h>
#include <sf/lua/LuaBindHelpers.h>

#include <sf/core/Settings.h>

#include <sf/graphics/Font.h>
#include <sf/graphics/renderer.h>

#include <sf/gui/effect.h>
#include <sf/gui/basewidget.h>
#include <sf/gui/widget.h>
#include <sf/gui/basewindow.h>
#include <sf/gui/window.h>
#include <sf/gui/imagewidget.h>
#include <sf/gui/labelwidget.h>
#include <sf/gui/buttonwidget.h>
#include <sf/gui/checkboxwidget.h>
#include <sf/gui/comboboxwidget.h>
#include <sf/gui/editwidget.h>
#include <sf/gui/listwidget.h>
#include <sf/gui/scrollwidget.h>
#include <sf/gui/sliderwidget.h>
#include <sf/gui/tablewidget.h>
#include <sf/gui/tabwidget.h>
#include <sf/gui/radiobuttongroupwidget.h>
#include <sf/gui/progressbarwidget.h>
#include <sf/gui/treewidget.h>
#include <sf/gui/guieffects.h>
#include <sf/gui/toolbarwidget.h>
#include <sf/gui/luawidget.h>
#include <sf/gui/TransformationWidget.h>
#include <sf/gui/TimeLineWidget.h>
#include <sf/gui/ToolTipWidget.h>
#include <sf/gui/ClipWidget.h>
#include <sf/gui/widgetmanager.h>

#include <sf/diag/assert.h>

namespace game
{
namespace lua
{

namespace 
{

template <typename T>
inline mluabind::i::CCl<sf::gui::Insets<T> >& BindInsets(const char* _typename)
{
	using namespace mluabind;
	using namespace sf::gui;

	typedef Insets<T> TypeT;

	return
		mluabind::Class<TypeT>(_typename)
			.Constructor()
			.template Constructor<T, T, T, T>()
			.LUA_CLASS_MEMBER(TypeT, Left)
			.LUA_CLASS_MEMBER(TypeT, Right)
			.LUA_CLASS_MEMBER(TypeT, Top)
			.LUA_CLASS_MEMBER(TypeT, Bottom)
		;
}


static sf::gui::CBaseWidget::WidgetPtrT BaseWidgetGetWidgetConst(const sf::gui::CBaseWidget* _this, const sf::IdString& _id)
{
	return _this->GetWidget(_id);
}

static sf::gui::CBaseWidget::WidgetPtrT BaseWidgetGetWidget(sf::gui::CBaseWidget* _this, const sf::IdString& _id)
{
	return _this->GetWidget(_id);
}

static int GetButtonStateInt(sf::gui::CButtonWidget* _button_widget)
{
	return _button_widget->GetButtonState();
}

static void CursorSetCursor(sf::gui::CCursor* _cursor, int _cursor_type)
{
	SFASSERT(_cursor_type>=0 && _cursor_type<sf::gui::CCursor::CursorCount);
	_cursor->SetCursor((sf::gui::CCursor::Cursors(_cursor_type)));
}

static int CursorGetCursor(sf::gui::CCursor* _cursor)
{
	return _cursor->GetCursor();
}
	
static bool SuperOnChar(sf::gui::CWindow* _this, int _char, int _keyboard_state, int _state, bool _broadcast)
{
	return _this->sf::gui::CBaseWindow::OnChar((wchar_t)_char, _keyboard_state, _state, _broadcast);
}

static bool SuperOnKeyDown(sf::gui::CWindow* _this, int _key, int _keyboard_state, int _state, bool _broadcast)
{
	return _this->sf::gui::CBaseWindow::OnKeyDown(_key, _keyboard_state, _state, _broadcast);
}

static bool SuperOnKeyUp(sf::gui::CWindow* _this, int _key, int _keyboard_state, int _state, bool _broadcast)
{
	return _this->sf::gui::CBaseWindow::OnKeyUp(_key, _keyboard_state, _state, _broadcast);
}

static bool SuperOnMouseMove(sf::gui::CWindow* _this, const sf::misc::IntVector& _pos, int _state, bool _broadcast)
{
	return _this->sf::gui::CBaseWindow::OnMouseMove(_pos, _state, _broadcast);
}

static bool SuperOnMouseDown(sf::gui::CWindow* _this, const sf::misc::IntVector& _pos, 
							 sf::gui::CWindow::MouseButtons _button, int _state, bool _broadcast)
{
	return _this->sf::gui::CBaseWindow::OnMouseDown(_pos, _button, _state, _broadcast);
}

static bool SuperOnMouseUp(sf::gui::CWindow* _this, const sf::misc::IntVector& _pos, sf::gui::CWindow::MouseButtons _button, int _state, bool _broadcast)
{
	return _this->sf::gui::CBaseWindow::OnMouseUp(_pos, _button, _state, _broadcast);
}

static bool SuperOnDoubleClick(sf::gui::CWindow* _this, const sf::misc::IntVector& _pos, int _state, bool _broadcast)
{
	return _this->sf::gui::CBaseWindow::OnDoubleClick(_pos, _state, _broadcast);
}

static bool SuperOnMouseWheel(sf::gui::CWindow* _this, const sf::misc::IntVector& _pos, int _delta, int _state, bool _broadcast)
{
	return _this->sf::gui::CBaseWindow::OnMouseWheel(_pos, _delta, _state, _broadcast);
}

static void SuperOnChildAction(sf::gui::CWindow* _this, const char* _action, sf::gui::CWidget* _child)
{
	_this->sf::gui::CBaseWindow::OnChildAction(_action, _child);
}
	
} //namespace


void BindGUI(sf::lua::CLua& _lua)
{
	using namespace mluabind;
	using namespace sf;
	using namespace sf::gui;

	_lua.StartBinding();

	_lua.Insert
	(
		Declare("sf")
			+ (Declare("gui")

				+ LUA_ENUM(JustifyLeft)
				+ LUA_ENUM(JustifyCenterH)
				+ LUA_ENUM(JustifyRight)
				+ LUA_ENUM(JustifyTop)
				+ LUA_ENUM(JustifyCenterV)
				+ LUA_ENUM(JustifyBottom)
//#ifndef __SF_IPHONE			   
				+ LUA_ENUM(SF_NONE)
				+ LUA_ENUM(SF_LEFT)
				+ LUA_ENUM(SF_RIGHT)
				+ LUA_ENUM(SF_UP)
				+ LUA_ENUM(SF_DOWN)
				+ LUA_ENUM(SF_HOME)
				+ LUA_ENUM(SF_END)
				+ LUA_ENUM(SF_PGUP)
				+ LUA_ENUM(SF_PGDOWN)
				+ LUA_ENUM(SF_INSERT)
				+ LUA_ENUM(SF_DELETE)
				+ LUA_ENUM(SF_SPACE)
				+ LUA_ENUM(SF_BACK)
				+ LUA_ENUM(SF_RETURN)
				+ LUA_ENUM(SF_TAB)
				+ LUA_ENUM(SF_ESC)
				+ LUA_ENUM(SF_F1)
				+ LUA_ENUM(SF_F2)
				+ LUA_ENUM(SF_F3)
				+ LUA_ENUM(SF_F4)
				+ LUA_ENUM(SF_F5)
				+ LUA_ENUM(SF_F6)
				+ LUA_ENUM(SF_F7)
				+ LUA_ENUM(SF_F8)
				+ LUA_ENUM(SF_F9)
				+ LUA_ENUM(SF_F10)
				+ LUA_ENUM(SF_F11)
				+ LUA_ENUM(SF_F12)
                + LUA_ENUM(SF_OEM_2)
 			    + LUA_ENUM(SF_KEYBOARD_REJECT)
			   
//#endif
			   
				//FloatInsets
				+ BindInsets<float>("FloatInsets")

				//IntInsets
//				+ BindInsets<int>("IntInsets")

				+ LUA_CLASS(CCursor)
					.LUA_CLASS_ENUM(CCursor, CursorPointer)
					.LUA_CLASS_ENUM(CCursor, CursorHand)
					.LUA_CLASS_ENUM(CCursor, CursorText)
					.LUA_CLASS_ENUM(CCursor, CursorHourglass)
					.LUA_CLASS_ENUM(CCursor, CursorCustom)
					.LUA_CLASS_ENUM(CCursor, CursorNone)
					.Method("SetCursor", &CursorSetCursor)
					.LUA_CLASS_METHOD(CCursor, SetCustomCursor)
					.Method("GetCursor", &CursorGetCursor)
					.LUA_CLASS_METHOD(CCursor, IsCustomCursorsEnabled)
					.LUA_CLASS_METHOD(CCursor, EnableCustomCursors)
					.LUA_CLASS_METHOD(CCursor, HideCursor)
					.LUA_CLASS_METHOD(CCursor, IsHidden)
					.LUA_CLASS_METHOD(CCursor, GetPosition)
				+ (Declare("g_Cursor")
					+ Function("Instance", &g_Cursor::Instance)
				)

				+ LUA_CLASS(CEffect)
					.Constructor()
					.LUA_CLASS_METHOD(CEffect, InitWidget)
					.LUA_CLASS_METHOD(CEffect, UpdateWidget)
					.LUA_CLASS_METHOD(CEffect, SetDead)
					.LUA_CLASS_METHOD(CEffect, IsDead)
				+ BIND_INTRUSIVE_PTR(CEffect)

				+ LUA_CLASS(CPositionChangeEffect)
					.Base<CEffect>()
					.Constructor<const misc::FloatVector&, float, float, core::CGroupTimer*>()
					.LUA_CLASS_METHOD(CPositionChangeEffect, InitWidget)
					.LUA_CLASS_METHOD(CPositionChangeEffect, UpdateWidget)
				+ BIND_INTRUSIVE_PTR(CPositionChangeEffect)

				+ LUA_CLASS(CColorChangeEffect)
					.Base<CEffect>()
					.Constructor<const graphics::Color&, float, float, core::CGroupTimer*>()
					.LUA_CLASS_METHOD(CColorChangeEffect, InitWidget)
					.LUA_CLASS_METHOD(CColorChangeEffect, UpdateWidget)
				+ BIND_INTRUSIVE_PTR(CColorChangeEffect)

                + LUA_CLASS(CWidgetManager)
					.LUA_CLASS_METHOD_OVERLOAD(CWidgetManager, GetKeyboardState, int (CWidgetManager::*)() const)

				+ LUA_CLASS(CBaseWidget)
					.Base<misc::CRefCountedObj>()
					.LUA_CLASS_ENUM(CBaseWidget, FlagHidden)
					.LUA_CLASS_ENUM(CBaseWidget, FlagDisabled)
					.LUA_CLASS_ENUM(CBaseWidget, FlagClipDraw)
					.LUA_CLASS_ENUM(CBaseWidget, FlagDead)
					.LUA_CLASS_ENUM(CBaseWidget, FlagDrawFloatRect)
					.LUA_CLASS_ENUM(CBaseWidget, LeftButton)
					.LUA_CLASS_ENUM(CBaseWidget, RightButton)
					.LUA_CLASS_ENUM(CBaseWidget, MiddleButton)
					.LUA_CLASS_ENUM(CBaseWidget, ShiftPressed)
					.LUA_CLASS_ENUM(CBaseWidget, ControlPressed)
					.LUA_CLASS_ENUM(CBaseWidget, AltPressed)
					.LUA_CLASS_METHOD(CBaseWidget, UpdateLayout)
					.LUA_CLASS_METHOD(CBaseWidget, UpdateLayoutOffsets)
					.LUA_CLASS_METHOD(CBaseWidget, Update)
					.LUA_CLASS_METHOD(CBaseWidget, Draw)
					.LUA_CLASS_METHOD(CBaseWidget, SetOffset)
					.LUA_CLASS_METHOD(CBaseWidget, GetOffset)
					.LUA_CLASS_METHOD(CBaseWidget, SetSize)
					.LUA_CLASS_METHOD(CBaseWidget, GetSize)
					.LUA_CLASS_METHOD(CBaseWidget, GetPreferredSize)
					.LUA_CLASS_METHOD(CBaseWidget, SetInsets)
					.LUA_CLASS_METHOD(CBaseWidget, GetInsets)
					.LUA_CLASS_METHOD(CBaseWidget, SetRotation)
					.LUA_CLASS_METHOD(CBaseWidget, GetRotation)
					.LUA_CLASS_METHOD(CBaseWidget, GetPoly)
					.LUA_CLASS_METHOD(CBaseWidget, SetColor)
					.LUA_CLASS_METHOD(CBaseWidget, GetColor)
					.LUA_CLASS_METHOD(CBaseWidget, SetChildFocused)
					.LUA_CLASS_METHOD(CBaseWidget, AddWidget)
					.LUA_CLASS_METHOD_OVERLOAD(CBaseWidget, RemoveWidget, bool (CBaseWidget::*)(CWidget*))
					.LUA_CLASS_METHOD_OVERLOAD(CBaseWidget, RemoveWidget, bool (CBaseWidget::*)(CBaseWidget::WidgetPtrT))
					.LUA_CLASS_METHOD(CBaseWidget, RemoveAllWidgets)
					.LUA_CLASS_METHOD_OVERLOAD(CBaseWidget, GetWidget, CBaseWidget::WidgetPtrT (CBaseWidget::*)(const IdString&, bool))
					.LUA_CLASS_METHOD_OVERLOAD(CBaseWidget, GetWidget, 
						const CBaseWidget::WidgetPtrT (CBaseWidget::*)(const IdString&, bool)const)
					.Method("GetWidget", &BaseWidgetGetWidget)
					.Method("GetWidget", &BaseWidgetGetWidgetConst)
					.LUA_CLASS_METHOD_OVERLOAD(CBaseWidget, GetWidgets, CBaseWidget::WidgetIteratorT (CBaseWidget::*)())
					.LUA_CLASS_METHOD_OVERLOAD(CBaseWidget, GetWidgets, CBaseWidget::ConstWidgetIteratorT (CBaseWidget::*)() const)
					.LUA_CLASS_METHOD(CBaseWidget, LayChildWidgetInFront)
					.LUA_CLASS_METHOD(CBaseWidget, AddEffect)
					.LUA_CLASS_METHOD(CBaseWidget, RemoveAllEffects)
					.LUA_CLASS_METHOD(CBaseWidget, GetId)
					.LUA_CLASS_METHOD(CBaseWidget, GetFlags)
					.LUA_CLASS_METHOD(CBaseWidget, AddFlags)
					.LUA_CLASS_METHOD(CBaseWidget, RemFlags)
					.LUA_CLASS_METHOD(CBaseWidget, SetFlags)
					.LUA_CLASS_METHOD(CBaseWidget, AdaptPos)
					.LUA_CLASS_METHOD(CBaseWidget, OnChildAction)
					.LUA_CLASS_METHOD(CBaseWidget, Load)
					.LUA_CLASS_METHOD(CBaseWidget, OnMouseDown)
					.LUA_CLASS_METHOD(CBaseWidget, OnMouseUp)
                    .LUA_CLASS_METHOD(CBaseWidget, AddToolTip)

				+ BIND_INTRUSIVE_PTR(CBaseWidget)

				+ LUA_CLASS(CWidget)
					.Base<CBaseWidget>()
					.LUA_CLASS_ENUM(CWidget, BroadcastOnChar)
					.LUA_CLASS_ENUM(CWidget, BroadcastOnKeyDown)
					.LUA_CLASS_ENUM(CWidget, BroadcastOnKeyUp)
					.LUA_CLASS_ENUM(CWidget, BroadcastOnMouseMove)
					.LUA_CLASS_ENUM(CWidget, BroadcastOnMouseDrag)
					.LUA_CLASS_ENUM(CWidget, BroadcastOnMouseDown)
					.LUA_CLASS_ENUM(CWidget, BroadcastOnMouseUp)
					.LUA_CLASS_ENUM(CWidget, BroadcastOnDoubleClick)
					.LUA_CLASS_ENUM(CWidget, BroadcastOnMouseWheel)
					.LUA_CLASS_ENUM(CWidget, AnchorLeft)
					.LUA_CLASS_ENUM(CWidget, AnchorRight)
					.LUA_CLASS_ENUM(CWidget, AnchorTop)
					.LUA_CLASS_ENUM(CWidget, AnchorBottom)
					.LUA_CLASS_ENUM(CWidget, LayerTop)
					.LUA_CLASS_ENUM(CWidget, LayerAlwaysOnTop)
					.Constructor<const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CWidget, SetAnchors)
					.LUA_CLASS_METHOD(CWidget, GetAnchors)
					.LUA_CLASS_METHOD(CWidget, GetBroadcastMask)
					.LUA_CLASS_METHOD(CWidget, GetChildrenBroadcastMask)
					.LUA_CLASS_METHOD(CWidget, GetLayer)
					.LUA_CLASS_METHOD(CWidget, IsFocused)
					.LUA_CLASS_METHOD(CWidget, LayInFront)
					.LUA_CLASS_METHOD(CWidget, SetLayer)
					.LUA_CLASS_METHOD(CWidget, GetAbsPos)
					.LUA_CLASS_METHOD_OVERLOAD(CWidget, GetParentWidget, CBaseWidget* (CWidget::*)())
					.LUA_CLASS_METHOD_OVERLOAD(CWidget, GetParentWidget, const CBaseWidget* (CWidget::*)() const)
				+ BIND_INTRUSIVE_PTR(CWidget)


				+ (Declare("CBaseWidget")
				   + sf::lua::BindGenericIterator(CBaseWidget::WidgetIteratorT(), "WidgetIteratorT")
				   + sf::lua::BindGenericIterator(CBaseWidget::ConstWidgetIteratorT(), "ConstWidgetIteratorT")
				)

				+ LUA_CLASS(CBaseWindow)
					.Base<CBaseWidget>()
					.Constructor<const IdString&>()
					.LUA_CLASS_METHOD(CBaseWindow, DoModal)
					.LUA_CLASS_METHOD(CBaseWindow, EndModal)
					.LUA_CLASS_METHOD(CBaseWindow, AddChildWindow)
					.LUA_CLASS_METHOD(CBaseWindow, RemoveChildWindow)
					.LUA_CLASS_METHOD(CBaseWindow, GetAbsPos)
					.LUA_CLASS_METHOD_OVERLOAD(CBaseWindow, GetParentWindow, CBaseWindow* (CBaseWindow::*)())
					.LUA_CLASS_METHOD_OVERLOAD(CBaseWindow, GetChildWindow, const CBaseWindow::WindowPtrT (CBaseWindow::*)() const)
					.LUA_CLASS_METHOD_OVERLOAD(CBaseWindow, GetChildWindow, CBaseWindow::WindowPtrT (CBaseWindow::*)())
   					.LUA_CLASS_METHOD(CBaseWindow, OnMouseMove)

				+ BIND_INTRUSIVE_PTR(CBaseWindow)

				
				+ LUA_CLASS(CWindow)
					.Base<CBaseWindow>()
					.Constructor()
					.Constructor<const graphics::CImage&, const IdString&>()
					.LUA_CLASS_METHOD(CWindow, SetCanDrag)
					.LUA_CLASS_METHOD(CWindow, IsCanDrag)
					.LUA_CLASS_METHOD(CWindow, HideToolTip)
					.Method("SuperOnChar", &SuperOnChar)
					.Method("SuperOnKeyDown", &SuperOnKeyDown)
					.Method("SuperOnKeyUp", &SuperOnKeyUp)
					.Method("SuperOnMouseMove", &SuperOnMouseMove)
					.Method("SuperOnMouseUp", &SuperOnMouseUp)
					.Method("SuperOnMouseDown", &SuperOnMouseDown)
					.Method("SuperOnDoubleClick", &SuperOnDoubleClick)
					.Method("SuperOnMouseWheel", &SuperOnMouseWheel)
					.Method("SuperOnChildAction", &SuperOnChildAction)
					//.Method("SuperDoDraw", &SuperDoDraw)
				+ BIND_INTRUSIVE_PTR(CWindow)

				+ LUA_CLASS(CImageWidget)
                    .LUA_CLASS_ENUM(CImageWidget, StretchMaximum)
                    .LUA_CLASS_ENUM(CImageWidget, StretchProportional)
                    .LUA_CLASS_ENUM(CImageWidget, StretchOnlyDecrease)
                    .LUA_CLASS_ENUM(CImageWidget, StretchNo)
					.Base<CWidget>()
					.Constructor<const TextureIdString&, const IdString&, unsigned, int>()
					.Constructor<const graphics::CImage&, const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD_OVERLOAD(CImageWidget, SetImage, void (CImageWidget::*)(const TextureIdString&))
					.LUA_CLASS_METHOD_OVERLOAD(CImageWidget, SetImage, void (CImageWidget::*)(const graphics::CImage&))
					.LUA_CLASS_METHOD_OVERLOAD(CImageWidget, GetImage, const graphics::CImage& (CImageWidget::*)()const)
					.LUA_CLASS_METHOD_OVERLOAD(CImageWidget, GetImage, graphics::CImage& (CImageWidget::*)())
                    .LUA_CLASS_METHOD(CImageWidget, SetStretchType)
                    .LUA_CLASS_METHOD(CImageWidget, GetStretchType)
				+ BIND_INTRUSIVE_PTR(CImageWidget)

				+ LUA_CLASS(CLabelWidget)
					.Base<CWidget>()
					.Constructor()
					.Constructor<const graphics::CFont*, const SF_WSTRING&, const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CLabelWidget, ResizeToFitText)
					.LUA_CLASS_METHOD(CLabelWidget, SetText)
					.LUA_CLASS_METHOD(CLabelWidget, GetText)
					.LUA_CLASS_METHOD(CLabelWidget, SetJustifications)
					.LUA_CLASS_METHOD(CLabelWidget, SetFontScale)
					.LUA_CLASS_METHOD(CLabelWidget, SetWordBreak)
					
				+ BIND_INTRUSIVE_PTR(CLabelWidget)

				+ LUA_CLASS(CClipWidget)
					.Base<CWidget>()
					.Constructor<const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CClipWidget, GetClip)
				+ BIND_INTRUSIVE_PTR(CClipWidget)

				+ LUA_CLASS(CButtonWidget)
					.Base<CWidget>()
					.LUA_CLASS_ENUM(CButtonWidget, StateNormal)
					.LUA_CLASS_ENUM(CButtonWidget, StateOver)
					.LUA_CLASS_ENUM(CButtonWidget, StatePressed)
					.LUA_CLASS_ENUM(CButtonWidget, StateDisabled)
					.Constructor<const graphics::CImage&, CButtonWidget::LabelPtrT, const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CButtonWidget, SetText)
					.LUA_CLASS_METHOD(CButtonWidget, GetText)
					.LUA_CLASS_METHOD(CButtonWidget, SetLabelColor)
					.LUA_CLASS_METHOD(CButtonWidget, GetButtonState)
					.Method("GetButtonStateInt", &GetButtonStateInt)
					.LUA_CLASS_METHOD(CButtonWidget, DisableButton)
					.LUA_CLASS_METHOD(CButtonWidget, OnButtonPressed)
					.LUA_CLASS_METHOD(CButtonWidget, OnMouseMove)
				+ BIND_INTRUSIVE_PTR(CButtonWidget)

				
				+ LUA_CLASS(CCheckboxWidget)
					.Base<CWidget>()
					.Constructor<const graphics::CImage&, const graphics::CImage&, const graphics::CFont*, const SF_WSTRING&, 
						const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CCheckboxWidget, Check)
					.LUA_CLASS_METHOD(CCheckboxWidget, IsChecked)
					.LUA_CLASS_METHOD(CCheckboxWidget, SetUnspecifiedState)
					.LUA_CLASS_METHOD(CCheckboxWidget, OnCheckChanged)
				+ BIND_INTRUSIVE_PTR(CCheckboxWidget)

/*				+ LUA_CLASS(CComboBoxWidget)
					.Base<CWidget>()
					.Constructor<const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CComboBoxWidget, ClearLines)
					.LUA_CLASS_METHOD(CComboBoxWidget, EnumLines)
					.LUA_CLASS_METHOD(CComboBoxWidget, AddLine)
					.LUA_CLASS_METHOD(CComboBoxWidget, GetLine)
					.LUA_CLASS_METHOD(CComboBoxWidget, FindLine)
					.LUA_CLASS_METHOD(CComboBoxWidget, SetLine)
					.LUA_CLASS_METHOD(CComboBoxWidget, RemoveLine)
					.LUA_CLASS_METHOD(CComboBoxWidget, SelectLine)
					.LUA_CLASS_METHOD(CComboBoxWidget, GetSelectedLine)
					.LUA_CLASS_METHOD(CComboBoxWidget, ShowList)
				+ BIND_INTRUSIVE_PTR(CComboBoxWidget)*/

				+ LUA_CLASS(CEditWidget)
					.Base<CWidget>()
					.Constructor<const graphics::CImage&, const graphics::CImage&, const SF_WSTRING&, 
						const graphics::CFont*, const FloatInsets&, const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CEditWidget, SetEditColors)
					.LUA_CLASS_METHOD(CEditWidget, SelectText)
					.LUA_CLASS_METHOD(CEditWidget, SetText)
					.LUA_CLASS_METHOD(CEditWidget, GetText)
					.LUA_CLASS_METHOD(CEditWidget, SetTextJustification)
					.LUA_CLASS_METHOD(CEditWidget, SetMaxChars)
					.LUA_CLASS_METHOD(CEditWidget, OnTextChanged)
				+ BIND_INTRUSIVE_PTR(CEditWidget)

				+ LUA_CLASS(CListWidget)
					.Base<CWidget>()
					.LUA_CLASS_ENUM(CListWidget, ListColorTextNormal)
					.LUA_CLASS_ENUM(CListWidget, ListColorTextSelected)
					.LUA_CLASS_ENUM(CListWidget, ListColorTextOver)
					.LUA_CLASS_ENUM(CListWidget, ListColorSelection)
					.Constructor<const graphics::CImage&, CListWidget::ScrollPtrT, const graphics::CFont*, const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CListWidget, CalcHeightToFitLines)
					.LUA_CLASS_METHOD(CListWidget, SetListHeight)
					.LUA_CLASS_METHOD(CListWidget, EnumLines)
					.LUA_CLASS_METHOD(CListWidget, ClearLines)
					.LUA_CLASS_METHOD(CListWidget, AddLine)
					.LUA_CLASS_METHOD(CListWidget, GetLine)
					.LUA_CLASS_METHOD(CListWidget, FindLine)
					.LUA_CLASS_METHOD(CListWidget, SetLine)
					.LUA_CLASS_METHOD(CListWidget, RemoveLine)
					.LUA_CLASS_METHOD(CListWidget, SelectLine)
					.LUA_CLASS_METHOD(CListWidget, GetSelectedLine)
					.LUA_CLASS_METHOD(CListWidget, GetScrollWidth)
					.LUA_CLASS_METHOD(CListWidget, SetLineJustifications)
					.LUA_CLASS_METHOD(CListWidget, SetListColors)
					.LUA_CLASS_METHOD(CListWidget, OnLineSelected)
				+ BIND_INTRUSIVE_PTR(CListWidget)

/*				+ LUA_CLASS(CScrollWidget)
					.Base<CWidget>()
					.Constructor<const graphics::CImage&, const graphics::CImage&, const graphics::CImage&, const graphics::CImage&, 
						bool, const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CScrollWidget, SetValue)
					.LUA_CLASS_METHOD(CScrollWidget, GetValue)
					.LUA_CLASS_METHOD(CScrollWidget, SetMaxValue)
					.LUA_CLASS_METHOD(CScrollWidget, GetMaxValue)
					.LUA_CLASS_METHOD(CScrollWidget, SetPageSize)
					.LUA_CLASS_METHOD(CScrollWidget, GetPageSize)
					.LUA_CLASS_METHOD(CScrollWidget, OnValueChanged)
                    .LUA_CLASS_METHOD(CScrollWidget, SetButtonValue)
				+ BIND_INTRUSIVE_PTR(CScrollWidget)*/

				+ LUA_CLASS(CSliderWidget)
					.Base<CWidget>()
					.Constructor<const graphics::CImage&, const graphics::CImage&, const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CSliderWidget, SetValue)
					.LUA_CLASS_METHOD(CSliderWidget, GetValue)
					.LUA_CLASS_METHOD(CSliderWidget, OnValueChanged)
				+ BIND_INTRUSIVE_PTR(CSliderWidget)

				+ LUA_CLASS(CTableWidget)
					.Base<CWidget>()
					.Constructor<const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD_OVERLOAD(CTableWidget, GetCell, CWidget* (CTableWidget::*)(int, int))
					.LUA_CLASS_METHOD_OVERLOAD(CTableWidget, LoadRow, 
						void (CTableWidget::*)(const core::CSettingsGroup& , const sf::core::CSettingsGroup&))
					.LUA_CLASS_METHOD(CTableWidget, FitCells)
					.LUA_CLASS_METHOD(CTableWidget, GetCellsCount)
					.LUA_CLASS_METHOD(CTableWidget, ClearTable)
					//.LUA_CLASS_METHOD(CTableWidget, LoadRow)
					.LUA_CLASS_METHOD(CTableWidget, SelectRow)
					.LUA_CLASS_METHOD(CTableWidget, GetSelectedRow)
				+ BIND_INTRUSIVE_PTR(CTableWidget)
 /*

				+ LUA_CLASS(CTabWidget)
					.Base<CWidget>()
					.Constructor<const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CTabWidget, EnumPanels)
					.LUA_CLASS_METHOD(CTabWidget, SelectPanel)
					.LUA_CLASS_METHOD(CTabWidget, GetSelectedPanel)
					.LUA_CLASS_METHOD_OVERLOAD(CTabWidget, GetPanelContent, CWidget* (CTabWidget::*)(int) const)
					.LUA_CLASS_METHOD_OVERLOAD(CTabWidget, GetPanelContent, CWidget* (CTabWidget::*)(const IdString&) const)
				+ BIND_INTRUSIVE_PTR(CTabWidget)
			   
				+ LUA_CLASS(CRadioButtonGroupWidget)
					.Base<CWidget>()
					.Constructor<const SF_WSTRING&, const SF_WSTRING&, const graphics::CFont*, const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CRadioButtonGroupWidget, GetChecked)
					.LUA_CLASS_METHOD(CRadioButtonGroupWidget, Check)
					.LUA_CLASS_METHOD(CRadioButtonGroupWidget, OnChangeChecked)
				+ BIND_INTRUSIVE_PTR(CRadioButtonGroupWidget)

				+ LUA_CLASS(CProgressBarWidget)
					.Base<CWidget>()
					.Constructor()
					.Constructor<const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CProgressBarWidget, SetImages)
					.LUA_CLASS_METHOD(CProgressBarWidget, SetValue)
				+ BIND_INTRUSIVE_PTR(CProgressBarWidget)
*/
				+ LUA_CLASS(CLuaWidget)
					.Base<CWidget>()
					.Constructor<const IdString&, unsigned, int>()
					.LUA_CLASS_METHOD(CLuaWidget, SetLua)
					.LUA_CLASS_METHOD(CLuaWidget, SetOverloadTable)
					.LUA_CLASS_METHOD(CLuaWidget, GetOverloadTable)
                    .LUA_CLASS_METHOD(CLuaWidget, BaseDoUpdate)
                    .LUA_CLASS_METHOD(CLuaWidget, BaseDoDraw)

/*				+ LUA_CLASS(CToolTipHelperWidget)
					.Base<CWidget>()
					.Constructor()
					.LUA_CLASS_METHOD(CToolTipHelperWidget, StopShowToolTip)
				+ BIND_INTRUSIVE_PTR(CToolTipHelperWidget)*/
				

			   )
	);

	_lua.FinishBinding();
}

} //lua
} //game
