#include <sf/sf.h>

#include <sf/lua/lua.h>
#include <sf/lua/LuaBindHelpers.h>

#include <sf/misc/misc.h>
#include <sf/misc/filewatcher.h>
#include <sf/misc/poly.h>
#include <sf/misc/vector.h>
#include <sf/misc/matrix.h>
#include <sf/misc/rect.h>
#include <sf/misc/stringtable.h>
#include <sf/misc/ptr.h>
#include <sf/misc/xmlwriter.h>
#include <sf/misc/location.h>
#include <sf/misc/math.h>
#include <sf/misc/sys/path.h>

#include <sf/misc/animation/clipobject.h>
#include <sf/misc/animation/imageobject.h>
#include <sf/misc/animation/particleobject.h>
#include <sf/misc/animation/pathobject.h>
#include <sf/misc/animation/path.h>
#include <sf/misc/animation/clip.h>
#include <sf/misc/animation/clipviewobject.h>
#include <sf/misc/animation/textobject.h>

#include <sf/diag/message.h>
#include <sf/diag/assert.h>

#include <sf/graphics/renderer.h>

#undef GetCurrentTime

namespace game
{
namespace lua
{

namespace 
{

static SF_WSTRING _FormatString(const wchar_t* _format, ...)
{
	const size_t g_BufferSize = 32768;
	va_list va;
	va_start(va, _format);
	wchar_t buffer[g_BufferSize] = L"";
	buffer[0] = 0;
	vswprintf(buffer, g_BufferSize, _format, va);
	va_end(va);
	return buffer;
}

static SF_WSTRING FormatString(const SF_WSTRING& _format, const SF_WSTRING& _p1) 
{ 
	return _FormatString(_format.c_str(), _p1.c_str()); 
}

static SF_WSTRING FormatString(const SF_WSTRING& _format, const SF_WSTRING& _p1, const SF_WSTRING& _p2) 
{ 
	return _FormatString(_format.c_str(), _p1.c_str(), _p2.c_str()); 
}

static SF_WSTRING FormatString(const SF_WSTRING& _format, int _p1) 
{ 
	return _FormatString(_format.c_str(), _p1); 
}

static SF_WSTRING FormatString(const SF_WSTRING& _format, int _p1, int _p2) 
{ 
	return _FormatString(_format.c_str(), _p1, _p2); 
}

static SF_WSTRING FormatString(const SF_WSTRING& _format, const SF_WSTRING& _p1, int _p2) 
{ 
	return _FormatString(_format.c_str(), _p1.c_str(), _p2); 
}

static SF_WSTRING FormatString(const SF_WSTRING& _format, int _p1, const SF_WSTRING& _p2) 
{ 
	return _FormatString(_format.c_str(), _p1, _p2.c_str()); 
}

#define BIND_FORMAT_STRING_1(T1) + Function("FormatString", static_cast<SF_WSTRING (*)(const SF_WSTRING&, T1)>(&FormatString))
#define BIND_FORMAT_STRING_2(T1, T2) + Function("FormatString", static_cast<SF_WSTRING (*)(const SF_WSTRING&, T1, T2)>(&FormatString))

static SF_WSTRING _t(const SF_WSTRING& _string)
{
	return sf::misc::ConvertString<SF_WSTRING>(_string);
}

static SF_WSTRING _t(const std::string& _string)
{
	return sf::misc::ConvertString<SF_WSTRING>(_string);
}

static SF_WSTRING _t(const char* _string)
{
	return sf::misc::ConvertString<SF_WSTRING>(_string);
}

static std::string t_(const SF_WSTRING& _string)
{
	return sf::misc::ConvertString<std::string>(_string);
}

static std::string t_(const std::string& _string)
{
	return _string;
}

static void MySetCurrentDirectory(const wchar_t* _dir)
{
	sf::SFSetCurrentDirectory(_dir);
}

static SF_WSTRING MyGetCurrentDirectory()
{
	wchar_t buff[256];
//	GetCurrentDirectory(MAX_PATH, buff);
	sf::SFGetCurrentDirectory(buff);

	return buff;
}

template <typename T>
static mluabind::i::CCl<sf::misc::Vector<T> >& BindVector(const char* _typename)
{
	using namespace mluabind;

	typedef sf::misc::Vector<T> TypeT;
	return
		mluabind::Class<TypeT>(_typename)
			.template Constructor<T, T>()
			.template Constructor<const sf::misc::FloatVector&>()
			.template Constructor<const sf::misc::IntVector&>()
			.Operator(Unm,  (TypeT (TypeT::*)() const) &TypeT::operator-)
			.Operator(Add,  (TypeT (TypeT::*)(const sf::misc::Vector<float>&) const) &TypeT::MEMBTEMPLATE operator+<float>)
			.Operator(Sub,  (TypeT (TypeT::*)(const sf::misc::Vector<float>&) const) &TypeT::MEMBTEMPLATE operator-<float>)
			.Operator(Add,  (TypeT (TypeT::*)(const sf::misc::Vector<int>&) const) &TypeT::MEMBTEMPLATE operator+<int>)
			.Operator(Sub,  (TypeT (TypeT::*)(const sf::misc::Vector<int>&) const) &TypeT::MEMBTEMPLATE operator-<int>)
			.Operator(Eq, &TypeT::MEMBTEMPLATE operator==<float>)
			.Operator(Eq, &TypeT::MEMBTEMPLATE operator==<int>)
//			.Operator(Add,  (TypeT (TypeT::*)(int) const) &TypeT::template operator+<int>)
//			.Operator(Sub,  (TypeT (TypeT::*)(float) const) &TypeT::template operator-<float>)
//			.Operator(Sub,  (TypeT (TypeT::*)(int) const) &TypeT::template operator-<int>)
//			.Operator(Sub, &TypeT::template operator-< misc::Vector<float> >)
//			.Operator(Sub, &TypeT::template operator-<float>)
//			.Operator(Add, &TypeT::template operator+<int>)
//			.Operator(Sub, &TypeT::template operator-<int>)
			.Operator(Mul, static_cast<TypeT (TypeT::*)(const sf::misc::FloatMatrix&) const>(&TypeT::operator*))
			.Operator(Mul, static_cast<TypeT (TypeT::*)(T) const>(&TypeT::operator*))
			.Operator(Div, static_cast<TypeT (TypeT::*)(T) const>(&TypeT::operator/))
			.LUA_CLASS_MEMBER(TypeT, X)
			.LUA_CLASS_MEMBER(TypeT, Y)
			.LUA_CLASS_METHOD(TypeT, ModSqr)
			.Method("Dot", &TypeT::template Dot<int>)
			.Method("Dot", &TypeT::template Dot<float>)
		;
}


template <typename T>
static mluabind::i::CCl<sf::misc::Rect<T> >& BindRect(const char* _typename)
{
	using namespace mluabind;

	typedef sf::misc::Rect<T> TypeT;

	return
		mluabind::Class<TypeT>(_typename)
			.template Constructor<T, T, T, T>()
			.template Constructor<const sf::misc::FloatRect&>()
			.template Constructor<const sf::misc::IntRect&>()
			.template Constructor<const sf::misc::Vector<T>&, const sf::misc::Vector<T>&>()
			.Method("IsIntersects", static_cast<bool (TypeT::*)(const TypeT&) const>(&TypeT::template IsIntersects<T>))
			.Method("GetIntersection", static_cast<TypeT (TypeT::*)(const TypeT&) const>(&TypeT::template GetIntersection<T>))
			.Method("GetUnion", static_cast<TypeT (TypeT::*)(const TypeT&) const>(&TypeT::template GetUnion<T>))
			.Method("IsContains", static_cast<bool (TypeT::*)(const TypeT&) const>(&TypeT::template IsContains<T>))
			.Method("IsContains", static_cast<bool (TypeT::*)(T, T) const>(&TypeT::template IsContains<T, T>))
			.LUA_CLASS_MEMBER(TypeT, X)
			.LUA_CLASS_MEMBER(TypeT, Y)
			.LUA_CLASS_MEMBER(TypeT, Width)
			.LUA_CLASS_MEMBER(TypeT, Height)
			.Operator(self==self)
		;
}

static float MatrixGetElementConst(const sf::misc::FloatMatrix* _this, size_t _row, size_t _col)
{
	return (*_this)(_row, _col);
}

static mluabind::i::CCl<sf::misc::FloatMatrix >& BindMatrix(const char* _typename)
{
	using namespace mluabind;
	
	typedef sf::misc::FloatMatrix TypeT;
	return
	mluabind::Class<TypeT>(_typename)
	.Constructor()
	.Constructor<float, float, float, float, float, float, float, float, float>()
	.Constructor<const TypeT&>()
	.Method("Elem", &MatrixGetElementConst)
	.Operator(Unm, (TypeT (TypeT::*)() const) &TypeT::operator-)
	.Operator(Add, (TypeT (TypeT::*)(const TypeT&) const) &TypeT::operator+)
	.Operator(Sub, (TypeT (TypeT::*)(const TypeT&) const) &TypeT::operator-)
	.Operator(Mul, static_cast<TypeT (TypeT::*)(const TypeT&) const >(&TypeT::operator*))	
	.Operator(Mul, static_cast<TypeT (TypeT::*)(float) const >(&TypeT::operator*))
	.Operator(Div, static_cast<TypeT (TypeT::*)(float) const >(&TypeT::operator/))
	;
}

int	BitwiseOr(const int _left_operand, const int _right_operand)
{
	return _left_operand | _right_operand;
}

int	BitwiseAnd(const int _left_operand, const int _right_operand)
{
	return _left_operand & _right_operand;
}

int	BitwiseXor(const int _left_operand, const int _right_operand)
{
	return _left_operand ^ _right_operand;
}

bool StringCompare(const SF_WSTRING& _a, const SF_WSTRING& _b)
{
	if((_a < _b)||(_a == _b))
	{
		return true;
	}
	else
	{
		return false;
	}
}

void MyMessageBoxA(const char* _text, const char* _header, sf_uint32 _type)
{
    SFASSERT(_type == 0);
    sf::diag::message::ShowA(_header, _text);
}

void MyMessageBoxW(const SF_WSTRING& _text, sf_uint32 _type)
{
    SFASSERT(_type == 0);
    sf::diag::message::ShowW(_text.c_str());
}

const sf::misc::FloatVector& PathPointControlPointsConst(const sf::misc::anim::CPath::Point* _this, size_t _p)
{
	static sf::misc::FloatVector dummy(0.f, 0.f);
	return _p>=2 ? dummy : _this->ControlPoints[_p];
}

sf::misc::FloatVector& PathPointControlPoints(sf::misc::anim::CPath::Point* _this, size_t _p)
{
	static sf::misc::FloatVector dummy(0.f, 0.f);
	return _p>=2 ? dummy : _this->ControlPoints[_p];
}

void CClipLoadByIdWithDef(sf::misc::anim::CClip* _this, const sf::IdString& _id)
{
	_this->Load(_id);
}

template <typename _Class, size_t _Number>
static mluabind::i::CCl<sf::String<_Class, _Number> >& BindString(const char* _typename)
{
using namespace mluabind;

	typedef sf::String<_Class, _Number> TypeT;

	return
		mluabind::Class<TypeT>(_typename)
			.Constructor()
			.template Constructor<const _Class*>()
			.Method("c_str", &TypeT::c_str)
			.Method("empty", &TypeT::empty)
			.Method("find", &TypeT::find)
			.Operator(Str, &TypeT::c_str)
			.Operator(const_self + const_self)
			.Operator(const_self == const_self)
			.Operator(const_self == other<const _Class*>())
		;
}

} //namespace

void BindMisc(sf::lua::CLua& _lua)
{
	using namespace mluabind;
	using namespace sf;	
	using namespace sf::lua;		
	using namespace sf::misc;
	using namespace sf::misc::anim;

	_lua.StartBinding();

	mluabind::BindStdLibrary(&_lua);

	_lua.Insert(
		Declare()
			//+ Function("_T", static_cast<SF_WSTRING(*)(const SF_WSTRING&)>(&_t))
			//+ Function("_T", static_cast<SF_WSTRING(*)(const std::string&)>(&_t))
			//+ Function("_T", static_cast<SF_WSTRING(*)(const char*)>(&_t))
			//+ Function("T_", static_cast<std::string (*)(const std::string&)>(&t_))
			//+ Function("T_", static_cast<std::string (*)(const SF_WSTRING&)>(&t_))
			BIND_FORMAT_STRING_1(const SF_WSTRING&)
			BIND_FORMAT_STRING_1(int)
			BIND_FORMAT_STRING_2(const SF_WSTRING&, const SF_WSTRING&)
			BIND_FORMAT_STRING_2(int, int)
			BIND_FORMAT_STRING_2(const SF_WSTRING&, int)
			BIND_FORMAT_STRING_2(int, const SF_WSTRING&)
	);

	_lua.Insert
	(
		Declare("sf")
			+ BindString<char, 24>		("StringA24")
			+ BindString<char, 32>		("StringA32")
			+ BindString<char, 40>		("StringA40")
			+ BindString<char, 56>		("StringA56")
			+ BindString<char, 88>		("StringA88")
			+ BindString<char, 120>		("StringA120")
			+ BindString<wchar_t, 28>	("StringW28")
			+ BindString<wchar_t, 60>	("StringW60")
			+ BindString<wchar_t, 124>	("StringW124")

			+ (Declare("misc")
				+ (Declare("anim")

					+ LUA_CLASS(CClipObject)
						.LUA_CLASS_ENUM(CClipObject, AnimationStatePlaying)
						.LUA_CLASS_ENUM(CClipObject, AnimationStatePaused)
						.LUA_CLASS_ENUM(CClipObject, AnimationStateStopped)
						.LUA_CLASS_ENUM(CClipObject, AnimationTypeOnce)
						.LUA_CLASS_ENUM(CClipObject, AnimationTypeLoop)
						.LUA_CLASS_ENUM(CClipObject, AnimationTypePingPong)
						.LUA_CLASS_ENUM(CClipObject, AnimationDirectionForward)
						.LUA_CLASS_ENUM(CClipObject, AnimationDirectionBackward)
						.LUA_CLASS_METHOD(CClipObject, Play)
						.LUA_CLASS_METHOD(CClipObject, Pause)
						.LUA_CLASS_METHOD(CClipObject, Stop)
                        .LUA_CLASS_METHOD(CClipObject, SetAnimationType)

                        .LUA_CLASS_METHOD(CClipObject, GetAnimationState)
                        .LUA_CLASS_METHOD(CClipObject, SetAnimationDirection)
                        .LUA_CLASS_METHOD(CClipObject, GetAnimationDirection)
                        .LUA_CLASS_METHOD(CClipObject, SetTime)
                        .LUA_CLASS_METHOD(CClipObject, GetTime)
                        .LUA_CLASS_METHOD(CClipObject, GetPeriodTime)
				   
						.LUA_CLASS_METHOD(CClipObject, GetSound)

						.LUA_CLASS_ENUM(CClipObject, FlipHorizontal)
						.LUA_CLASS_ENUM(CClipObject, FlipVertical)
						.LUA_CLASS_METHOD(CClipObject, Reload)
						.LUA_CLASS_METHOD(CClipObject, Load)
						.LUA_CLASS_METHOD(CClipObject, Draw)
						.LUA_CLASS_METHOD(CClipObject, Update)
						.LUA_CLASS_METHOD(CClipObject, GetId)
						.LUA_CLASS_METHOD(CClipObject, SetPos)
						.LUA_CLASS_METHOD(CClipObject, GetPos)
						.LUA_CLASS_METHOD(CClipObject, GetPoly)
						.LUA_CLASS_METHOD(CClipObject, SetPoly)
						.LUA_CLASS_METHOD(CClipObject, ResetPoly)
						.LUA_CLASS_METHOD(CClipObject, SetAngle)
						.LUA_CLASS_METHOD(CClipObject, GetAngle)
						.LUA_CLASS_METHOD(CClipObject, SetColor)
						.LUA_CLASS_METHOD(CClipObject, GetColor)
						.LUA_CLASS_METHOD(CClipObject, SetAdditiveBlend)
						.LUA_CLASS_METHOD(CClipObject, IsAdditiveBlend)
						.LUA_CLASS_METHOD(CClipObject, GetFlipFlags)
						.LUA_CLASS_METHOD(CClipObject, AddFlipFlags)
						.LUA_CLASS_METHOD(CClipObject, RemFlipFlags)
						.LUA_CLASS_METHOD(CClipObject, SetFlipFlags)
						.LUA_CLASS_METHOD(CClipObject, GetCustomFlags)
						.LUA_CLASS_METHOD(CClipObject, AddCustomFlags)
						.LUA_CLASS_METHOD(CClipObject, RemCustomFlags)
						.LUA_CLASS_METHOD(CClipObject, SetCustomFlags)
						.LUA_CLASS_METHOD(CClipObject, GetLocation)
						.LUA_CLASS_METHOD(CClipObject, IsContains)
						.LUA_CLASS_METHOD(CClipObject, SnapToPath)
						.LUA_CLASS_METHOD(CClipObject, IsSnapToPath)
						.LUA_CLASS_METHOD(CClipObject, SetPath)
						.LUA_CLASS_METHOD(CClipObject, GetPath)
						.LUA_CLASS_METHOD(CClipObject, SetUserFlags)
						.LUA_CLASS_METHOD(CClipObject, GetUserFlags)
						.LUA_CLASS_METHOD(CClipObject, SetVisible)
						.LUA_CLASS_METHOD(CClipObject, IsVisible)
						.LUA_CLASS_METHOD(CClipObject, GetType)
						.LUA_CLASS_METHOD(CClipObject, GetState)
						.LUA_CLASS_METHOD(CClipObject, GetResourceId)
						.LUA_CLASS_METHOD(CClipObject, DisableSounds)
                        .LUA_CLASS_METHOD(CClipObject, StopAllSounds)

//					+ (Declare("CClipObject")
//						+ mluabind::Class<CClipObject::LoadPathHelper>("LoadPathHelper")
//					)

					+ LUA_CLASS(CImageObject)
						.Base<CClipObject>()
						.Constructor()
						.Constructor<const char*>()
						.LUA_CLASS_METHOD(CImageObject, SetImage)
						.LUA_CLASS_METHOD(CImageObject, GetImageId)
//					+ (Declare("CImageObject")
//						+ Function("GetClassType", CImageObject::GetClassType)
//					)

					+ LUA_CLASS(CClipViewObject)
						.Base<CClipObject>()
						.Constructor()
						.Constructor<const IdString&>()
						.LUA_CLASS_METHOD(CClipViewObject, SetClip)
						.LUA_CLASS_METHOD(CClipViewObject, GetClipId)
//					+ (Declare("CClipViewObject")
//						+ Function("GetClassType", CClipViewObject::GetClassType)
//					)

					+ LUA_CLASS(CClip)
						.Constructor()
                        .Constructor<const CClip&>()
						.LUA_CLASS_ENUM(CClip, FlipHorizontal)
						.LUA_CLASS_ENUM(CClip, FlipVertical)
						.LUA_CLASS_METHOD(CClip, Update)
						.LUA_CLASS_METHOD(CClip, Draw)
						.LUA_CLASS_METHOD(CClip, GetViewSize)
						.LUA_CLASS_METHOD(CClip, SetPos)
						.LUA_CLASS_METHOD(CClip, GetPos)
						.LUA_CLASS_METHOD(CClip, SetSize)
						.LUA_CLASS_METHOD(CClip, GetSize)
						.LUA_CLASS_METHOD(CClip, SetAngle)
						.LUA_CLASS_METHOD(CClip, GetAngle)
						.LUA_CLASS_METHOD(CClip, SetColor)
						.LUA_CLASS_METHOD(CClip, GetColor)
						.LUA_CLASS_METHOD(CClip, DisableSounds)
						.LUA_CLASS_METHOD(CClip, Play)
						.LUA_CLASS_METHOD(CClip, Stop)
						.LUA_CLASS_METHOD(CClip, Pause)
						.LUA_CLASS_METHOD(CClip, SetTime)
                        .LUA_CLASS_METHOD(CClip, MoveTime)
                        .LUA_CLASS_METHOD(CClip, FindObject)
						.LUA_CLASS_METHOD(CClip, Load)
						.Method("Load", CClipLoadByIdWithDef)
//					+ (Declare("CClip")
//						+ Function("ReloadClips", &CClip::ReloadClips)
//					)
				)

				+ LUA_FUNCTION(StringCompare)
				+ LUA_FUNCTION(BitwiseOr)
				+ LUA_FUNCTION(BitwiseAnd)
				+ LUA_FUNCTION(BitwiseXor)


				+ LUA_FUNCTION(AppendPath)
				+ LUA_FUNCTION(CheckExistPath)
//				+ LUA_FUNCTION(CalcFileCRC)
				+ LUA_FUNCTION(SetNamedFullAccess)
//				+ LUA_FUNCTION(Compress)
//				+ LUA_FUNCTION(Uncompress)
				+ Function("SetCurrentDirectory", MySetCurrentDirectory)
				+ Function("GetCurrentDirectory", MyGetCurrentDirectory)

//				+ LUA_FUNCTION(Poly4ConvertToLocalPos)
//				+ LUA_FUNCTION(Poly4ConvertToAbsPos)

				+ BindVector<float>("FloatVector")
				+ BindVector<int>("IntVector")
				+ BindRect<float>("FloatRect")
				+ BindRect<int>("IntRect")
                + BindRect<short>("ShortRect")
			
				+ LUA_FUNCTION(CalculateFloatVectorModule)

				+ LUA_CLASS(Poly4)
					.Constructor()
					.Constructor<const Poly4&>()
					.LUA_CLASS_METHOD_OVERLOAD(Poly4, IsContains, bool (Poly4::*)(float, float) const)
					.LUA_CLASS_METHOD_OVERLOAD(Poly4, Point, const FloatVector& (Poly4::*)(size_t) const)
					.LUA_CLASS_METHOD_OVERLOAD(Poly4, Point, FloatVector& (Poly4::*)(size_t))
					.LUA_CLASS_METHOD(Poly4, GetBoundingRect)
					.LUA_CLASS_METHOD(Poly4, IsIntersects)
					.LUA_CLASS_METHOD(Poly4, Scale)
					.Operator(self + other<FloatVector>())
					.Operator(self + other<IntVector>())
					.Operator(self - other<FloatVector>())
					.Operator(self - other<IntVector>())
					.Operator(self * other<FloatMatrix>())
					.Operator(self * other<float>())
					.Operator(self==other<Poly4>())

				+ Function("Poly4FromRect", &Poly4FromRect<float>)
				+ Function("Poly4FromPoints", &Poly4FromPoints<float>)

				+ BindMatrix("FloatMatrix")
				+ LUA_FUNCTION(MatrixIdentity)
				+ LUA_FUNCTION_OVERLOAD(MatrixRotation, FloatMatrix (*)(float))
				+ LUA_FUNCTION_OVERLOAD(MatrixRotation, FloatMatrix (*)(float, float, float))
				+ LUA_FUNCTION(MatrixTranslation)
				+ LUA_FUNCTION_OVERLOAD(MatrixScale, FloatMatrix (*) (float, float))
                + LUA_FUNCTION_OVERLOAD(MatrixScale, FloatMatrix (*) (float, float, float, float))
				+ LUA_FUNCTION(MatrixInvert)

				+ LUA_CLASS(Location)
					.Constructor()
					.Constructor<const FloatVector&, const Poly4&, float>()
					.Constructor<const Location&>()
					.Constructor<const Poly4&>()
					.LUA_CLASS_METHOD(Location, IsContains)
					.LUA_CLASS_METHOD(Location, Scale)
					.LUA_CLASS_METHOD(Location, GetPolyPos)
					.LUA_CLASS_METHOD(Location, GetAbsPos)
					.LUA_CLASS_METHOD(Location, GetPoly)
					.LUA_CLASS_MEMBER(Location, Pos)
					.LUA_CLASS_MEMBER(Location, Contour)
					.LUA_CLASS_MEMBER(Location, Angle)

/*				+ LUA_CLASS(CFileWatcher)
					.Constructor<bool, unsigned>()
					.LUA_CLASS_METHOD(CFileWatcher, WatchFile)
					.LUA_CLASS_METHOD(CFileWatcher, ClearWatches)
					.LUA_CLASS_METHOD(CFileWatcher, IsFilesChanged)
					.LUA_CLASS_METHOD(CFileWatcher, GetChangedFiles)
					.LUA_CLASS_METHOD(CFileWatcher, ResetChanges)*/

				+ LUA_CLASS(CStringTable)
					.Constructor()
//					.LUA_CLASS_METHOD(CStringTable, SetFileName)
//					.LUA_CLASS_METHOD(CStringTable, CheckChanges)
					.LUA_CLASS_METHOD(CStringTable, GetString)
					.LUA_CLASS_METHOD_OVERLOAD(CStringTable, FormatByStringsID, SF_WSTRING (CStringTable::*)(const SF_WSTRING&) const )
					.LUA_CLASS_METHOD_OVERLOAD(CStringTable, FormatByStringsID, SF_WSTRING (CStringTable::*)(const std::string&) const )
					.LUA_CLASS_METHOD_OVERLOAD(CStringTable, FormatByStringsID, SF_WSTRING (CStringTable::*)(const char*) const )

				+ (Declare("g_StringTable")
					+ Function("Instance", &g_StringTable::Instance)
				)

/*				+ LUA_CLASS(CLocalString)
					.Constructor()
					.Constructor<const SF_WSTRING&>()
					.Constructor<const CLocalString&>()
					.LUA_CLASS_METHOD(CLocalString, c_str)*/
					//.Operator(Eq, static_cast<bool (*)(const CLocalString&, const CLocalString&)>(&misc::operator==))
					//.Operator(Eq, static_cast<bool (*)(const CLocalString&, const Char*)>(&misc::operator==))
					//.def(other<const wchar_t*>()==const_self)

				+ LUA_CLASS(CRefCountedObj)

				+ LUA_CLASS(CXmlWriter)
					.Constructor()
					.Constructor<bool>()
					.Constructor<bool, bool>()
					.LUA_CLASS_METHOD(CXmlWriter, SaveToFile)
					.LUA_CLASS_METHOD_OVERLOAD(CXmlWriter, Write, void (CXmlWriter::*)(const char*, int))
					.LUA_CLASS_METHOD_OVERLOAD(CXmlWriter, Write, void (CXmlWriter::*)(const wchar_t*))

				+ LUA_CLASS(CXmlElement)
					.Constructor<CXmlElement&, const char*>()
					.Constructor<CXmlWriter&, const char*>()
					.LUA_CLASS_METHOD(CXmlElement, Flush)
					.LUA_CLASS_METHOD_OVERLOAD(CXmlElement, Attr, void (CXmlElement::*)(const char*, const char*))

				+ Function("MessageBox", &MyMessageBoxA)
				+ Function("MessageBoxW", &MyMessageBoxW)

				+ LUA_FUNCTION(GetOrthogonalPos)
				+ LUA_FUNCTION(GetLinesIntersect)
				+ LUA_FUNCTION(GetBezierPos)

				+ BindPair<bool,float>("BoolFloatPair") 
				+ LUA_FUNCTION(GetAngle)
			)
	);

	_lua.FinishBinding();
}

} //lua
} //game
