#include "InAppBilling.h"

#include <sf/diag/assert.h>
#include <sf/diag/log.h>
#include <sf/core/application.h>
#include <sf/misc/sys/filewrp.h>

namespace game
{

// single instance of CInAppBilling
static CInAppBilling* g_Instance = 0;

// callbacks from Java
CInAppBilling& CInAppBilling::Instance()
{
    if (!g_Instance) g_Instance = new CInAppBilling;
    return *g_Instance;
}

void CInAppBilling::Load()
{
    wchar_t path[512];
    wcscpy(path, sf::core::g_Application->GetAppDataFolder().c_str());
    wcscat(path, L"/data");

    FILE* f = sf::SFfopen(path, L"rb");
    if (!f) return;
    
    int count = ItemsCount;
    fread(&count, sizeof(count), 1, f);
    SFASSERT(count <= ItemsCount);
    if (count > ItemsCount) return;
        
    for (int i = 0; i < count; ++i)
    {
        int state = 0;
        fread(&state, sizeof(state), 1, f);
        if (state == StatePending) state = StateAvailable;
        m_Items[i].State = (PurchaseState)state;
        
        char delivered = 0;
        fread(&delivered, sizeof(delivered), 1, f);
        m_Items[i].Delivered = (delivered != 0);
    }
    
    fclose(f);
}
    
bool CInAppBilling::IsServiceAvailable() const
{
    return true;
}
 
bool CInAppBilling::IsServiceConnected() const
{
    return true;
}

bool CInAppBilling::IsBillingSupported() const
{
    return true;
}

bool CInAppBilling::InitiatePurchase(const char * _purchaseId)
{
	SFASSERT(m_Items[0].State == StateAvailable);
    m_Items[0].State = StatePending;
	m_thread = CreateThread(NULL, 0, BillingThread, this, 0, NULL);
    return true;
}
 
CInAppBilling::PurchaseState CInAppBilling::GetItemPurchaseState(const char * _purchaseId) const
{
    return m_Items[0].State;
}

bool CInAppBilling::PurchaseDelivered(const char * _purchaseId)
{
    m_Items[0].Delivered = true;	
	Save();	
	return true;
}

bool CInAppBilling::IsPurchaseDelivered(const char * _purchaseId) const
{
    return m_Items[0].Delivered;
}

CInAppBilling::CInAppBilling()
{
	SFASSERT(!g_Instance);
	memset(m_Items, 0, ItemsCount*sizeof(Item));   
    // init items
    m_Items[ItemUnlockFullGame].Id = "android.test.purchased";
    			//"com.alawar.twisted_lands.full_game_unlock";
}

CInAppBilling::~CInAppBilling()
{
}

void CInAppBilling::Save()
{
    wchar_t path[512];
    wcscpy(path, sf::core::g_Application->GetAppDataFolder().c_str());
    wcscat(path, L"/data");

    FILE* f = sf::SFfopen(path, L"wb");
    SFASSERT(f);
    if (!f) return;
    
    int val = ItemsCount;
    fwrite(&val, sizeof(val), 1, f);
    for (int i = 0; i < ItemsCount; ++i)
    {
        val = m_Items[i].State;
        fwrite(&val, sizeof(val), 1, f);
        char delivered = m_Items[i].Delivered;
        fwrite(&delivered, sizeof(delivered), 1, f);
    }
    
    fclose(f);
}

DWORD WINAPI CInAppBilling::BillingThread(void *arg)
{
	CInAppBilling * b = (CInAppBilling*)arg;
	b->Billing();
	return 0;
}

void CInAppBilling::Billing()
{
	if(::MessageBox(NULL, L"Want you buy game?", L"Billing", MB_OKCANCEL) == IDOK)
	{
		Sleep(2000);
		m_Items[ItemUnlockFullGame].State = StatePurchased;
		::MessageBox(NULL, L"Thank you for purchasind", L"Billing", MB_OK);
	}
	else{
		m_Items[ItemUnlockFullGame].State = StateAvailable;
	}
}

} // game
