#pragma once

#include <sf/sf.h>
#include <windows.h>

namespace game
{

// Singleton that implements in app purchase mechanism.
class CInAppBilling
{
public:
    // list of all items available for purchase
    enum ItemId
    {
        ItemUnlockFullGame = 0,
        
        ItemsCount
    };
    // state of the purchaseable item
    enum PurchaseState
    {
        StateAvailable, // item is available for purchase
        StatePending,   // item is in the process of purchasing
        StatePurchased  // item is purchased
    };
    
    static CInAppBilling& Instance(); // creating using new, never release
    
    ~CInAppBilling();

    // loads data about purchases; call thisfrom CApp::Init
    void Load();
    
    // is service available for connection; if not, then billing is not supported at all
    bool IsServiceAvailable() const; 
    // is app connected to the billing service? if not and service is available, 
    //   then it is a good idea to wait for the app to connect to the service
    bool IsServiceConnected() const;
    // is billing supported by installed version of Android Market
    // billing support is discovered after the app successfully connects to the billing service
    bool IsBillingSupported() const;
    
	bool IsOnline() const { return true; }
    // initates purchase process
    bool InitiatePurchase(const char * _purchaseId);
    // returns saved purchase state of an item
    PurchaseState GetItemPurchaseState(const char * _purchaseId) const;
    // reports that the app successfully handled item purchase
    bool PurchaseDelivered(const char * _purchaseId);
    // returns true if purchase was delivered
    bool IsPurchaseDelivered(const char * _purchaseId) const;
        
private:
	static DWORD WINAPI BillingThread(void *arg);
	void Billing();

    // purchase events that come from java side of InAppBilling
	enum PurchaseEvents
    {
		EventCancel = 0, 
		EventError = 1,
		EventPurchase = 2, 
        EventRefund = 3
	};
    struct Item
    {
        inline Item() : Id(0), State(StateAvailable), Delivered(false) {}
        
        const char* Id;
        PurchaseState State;
        bool Delivered;
    };
    
    // singleton
    CInAppBilling();
    
    void CheckPurchases();
    void Save();
    
    Item m_Items[ItemsCount];
	HANDLE m_thread;
};

} //game