#include "ResourcesDownloader.h"
#include <sf/diag/log.h>

namespace game
{

// single instance of CResourcesDownloader
static CResourcesDownloader* g_Instance = 0;

CResourcesDownloader& CResourcesDownloader::Instance()
{
    if (!g_Instance) g_Instance = new CResourcesDownloader;
    return *g_Instance;
}

CResourcesDownloader::CResourcesDownloader()
	: m_length(1000)
	, m_downloading(false)
{
}

CResourcesDownloader::~CResourcesDownloader()
{
}

bool CResourcesDownloader::DownloadFile(const char* _source_link)
{
	if(IsDownloading())
		return false;

	m_downloading = true;
	DWORD id;
	m_thread = CreateThread(NULL, 0, &DownloadThread, this, 0, &id);
	return m_thread != NULL;
}

bool CResourcesDownloader::IsDownloading()
{
	EnterCriticalSection(&m_cs);
    bool ret = m_downloading;
	LeaveCriticalSection(&m_cs);

	return ret;
}

void CResourcesDownloader::StopDownload()
{
	EnterCriticalSection(&m_cs);
	m_downloading = false;
	LeaveCriticalSection(&m_cs);

	WaitForSingleObject(m_thread, INFINITE);
}

void CResourcesDownloader::ResetDownload(const char* _source_link)
{
    StopDownload();
}
    
int CResourcesDownloader::GetContentLength()
{
	EnterCriticalSection(&m_cs);
    int ret = m_length;
	LeaveCriticalSection(&m_cs);

	return ret;
}

int CResourcesDownloader::GetDownloadedLength()
{
	EnterCriticalSection(&m_cs);
    int ret = m_downloaded;
	LeaveCriticalSection(&m_cs);

	return ret;
}

DWORD WINAPI CResourcesDownloader::DownloadThread(void *arg)
{
	CResourcesDownloader * rd = (CResourcesDownloader *)arg;
	rd->Download();
	return 0;
}

void CResourcesDownloader::Download()
{
	m_downloaded = 0;
	while(1)
	{
		EnterCriticalSection(&m_cs);
		if(!m_downloading || (m_downloaded == m_length)){
			m_downloading = false;
			LeaveCriticalSection(&m_cs);
			break;
		}
		m_downloaded++;
		LeaveCriticalSection(&m_cs);
		Sleep(1);
	}
}

} // game
